
#include "Analysis.h"

void Analysis(std::string filename, std::string method)
{
  TCanvas* canvas = new TCanvas("canvas","",200,10,900,600);
  std::deque<TVector3> momentum,  momentumcp;

  {
    TFile* file = TFile::Open(filename.c_str(), "READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Koyomi",tree);
    tree->Print();

    TFile* workfile = TFile::Open("root/temp.root","recreate");
    if(!workfile) return;

    TTree* newtree = new TTree("Hitagi", "title");
    new TH1D("hamin", "title", 100, 0., 20.);

    CDataHimeSim* data = 0;
    TBranch *bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata) == TTree::kMissingBranch) return;

    CDataHime* expdata = new CDataHime;
    //     newtree->Branch("hime", &expdata, 32000, 0);
    newtree->Branch("hime", &expdata);

    Long64_t nentries = tree->GetEntriesFast();
    nentries = 100000;
    for(Long64_t i=0; i<nentries; i++) {
      std::cout << "\r event:" << i << std::flush;

      expdata->Reset();

      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      SimToExp(data, expdata);
      if(method == "Track"){
	Track(expdata);
      }else if(method == "Simple"){
	Simple(expdata);
      }else if(method == "Niren"){
	Niren(expdata);
      }else{
	std::cout << "method:" << method << " cannot use" << std::endl;
	return;
      }

      if(!expdata->fEventFlag) continue;

      momentum.push_back(data->fMomentum);
      momentumcp.push_back(data->fMomentumChargedParticle);

      newtree->Fill();
    }
    std::cout << std::endl;

    tree->ResetBranchAddresses();
    
    workfile->Write();
    workfile->Close();
    file->Close();
  }

  {
    TFile* file = TFile::Open("root/temp.root","READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Hitagi",tree);
    tree->Print();

    CDataHime* data = 0;
    TBranch *bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata) == TTree::kMissingBranch) return;

    TH1D* hpx = new TH1D("hpx", "title", 100, 0.-10, 0.+10);
    TH1D* hpy = new TH1D("hpy", "title", 100, 0.-10, 0.+10);
    TH1D* hpz = new TH1D("hpz", "title", 100, 0.-10, 0.+10);
    TH1D* herel = new TH1D("herel", "title", 100, 0., 1.);

    Long64_t nentries = tree->GetEntriesFast();
    std::deque<TVector3>::iterator it = momentum.begin();
    std::deque<TVector3>::iterator itcp = momentumcp.begin();
    for(Long64_t i=0; i<nentries; i++) {
      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      if(!data->fEventFlag){
	++it;
	++itcp;
	continue;
      }

      hpx->Fill(data->fP.Px()-it->Px());
      hpy->Fill(data->fP.Py()-it->Py());
      hpz->Fill(data->fP.Pz()-it->Pz());

      double u(931.494);

      int a1(1), a2(9);
      double p1_me(8.0713171);
      double p2_me(24.954264);
      CParticle p1(p1_me+u*a1),p2(p2_me+u*a2);

      static TRandom3 random;
      //      p1.p(it->Px(), it->Py(), it->Pz());
      p1.p(data->fP.Px(), data->fP.Py(), data->fP.Pz());
      //      p2.p(itcp->Px(), itcp->Py(), itcp->Pz());      
      p2.p(random.Gaus(itcp->Px(), itcp->Mag()/700.),
	   random.Gaus(itcp->Py(), itcp->Mag()/700.),
	   random.Gaus(itcp->Pz(), itcp->Mag()/700.));

      static CKinema kinema;
      double erel = kinema.InvariantMass(p1, p2) - p1.m() - p2.m();

      herel->Fill(erel);
      
      ++it;
      ++itcp;
    }

    canvas->Divide(3,2);
    canvas->cd(1);
    TH1D* hamin;
    gDirectory->GetObject("hamin", hamin);
    hamin->Draw();
    canvas->cd(2);
    hpx->Draw();
    hpx->Fit("gaus");
    canvas->cd(3);
    hpy->Draw();
    hpy->Fit("gaus");
    canvas->cd(4);
    hpz->Draw();
    hpz->Fit("gaus");
    canvas->cd(5);
    herel->Draw();

    canvas->Update();

    tree->ResetBranchAddresses();
  }
}

void SimToExp(CDataHimeSim* simdata, CDataHime* expdata)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.+itsim->fPosition.X()-shift;
    Double_t xr = length/2.-itsim->fPosition.X()+shift;
    Double_t yl = length/2.+itsim->fPosition.Y();
    Double_t yr = length/2.-itsim->fPosition.Y();

    Double_t fEdepee = MeVtoMeVee(itsim->fKineticEnergy,
				  itsim->fKineticEnergy - itsim->fEdep,
				  itsim->fParticleName.Data());
    //    std::cout << itsim->fEdep << " " << fEdepee << " " << itsim->fParticleName << std::endl;

    Double_t tl, tr, al, ar;
    if(id[0]%2){
      tl = itsim->fTime + yl/vl;
      tr = itsim->fTime + yr/vl;
      al = fEdepee*exp(-yl/attlength);
      ar = fEdepee*exp(-yr/attlength);
      //      al = itsim->fEdep*exp(-yl/attlength);
      //      ar = itsim->fEdep*exp(-yr/attlength);
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      al = fEdepee*exp(-xl/attlength);
      ar = fEdepee*exp(-xr/attlength);
      //      al = itsim->fEdep*exp(-xl/attlength);
      //      ar = itsim->fEdep*exp(-xr/attlength);
    }

    CDataHime::OneData* onedata = expdata->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      expdata->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    it->fTL += gRandom->Gaus(0,treso);  
    it->fTR += gRandom->Gaus(0,treso);  

    it->fT = (it->fTL+it->fTR-length/vl)/2;
    it->fE = sqrt(it->fAL*it->fAR)*exp(length/(2.*attlength));

    Double_t x;
    if(it->fId[0]%2){
      x = it->fId[1]*width + width/2. - efflength/2. + shift;
    }else{
      x = it->fId[1]*width + width/2. - efflength/2.;
    }
    Double_t y;
    if(it->fId[0]%2){
      y = vl*(it->fTL-it->fTR)/2.;
    }else{
      y = vl*(it->fTL-it->fTR)/2. + shift;
    }
    Double_t z = it->fId[0]*2.*depth+zdepth+1;

    if(it->fId[0]%2){
      it->fX.SetX(x);
      it->fX.SetY(y);
    }else{
      it->fX.SetX(y);
      it->fX.SetY(x);
    }
    it->fX.SetZ(z);
    ++it;
  }
}

void Track(CDataHime* expdata)
{
  TGraph2DErrors* graph = new TGraph2DErrors;

  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(it->fE < thr){
      it = expdata->fDequeOfOneData.erase(it);
    }else{
      ++it;
    }
  }

  if(expdata->fDequeOfOneData.size() < multihit){
    expdata->fEventFlag = 0;
    return;
  }

  Int_t point = 0;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
    if(it->fId[0]%2){
      graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
    }else{	 
      graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
    }

    ++point;
    ++it;
  }

  double parFit[4];
  double amin = line3Dfit(graph, parFit);

  TH1D* hamin;
  gDirectory->GetObject("hamin", hamin);
  hamin->Fill(amin);

  //ここがものすごく効く->つまり変なのが混じりすぎ->MeVee変換必須
  //もしくは選択的にトラックを引くアルゴリズムが必要。
  //たとえばamin<5になるまで、計算に入れる点を変えるとか。
  if(amin > aminmin){
    expdata->fEventFlag = 0;
    return;
  }

  std::deque<CDataHime::OneData>::iterator itmin;
  itmin = expdata->fDequeOfOneData.begin();
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  //analysis of timing
  double t1 = zdepth, x1, y1, z1;
  line(t1,parFit,x1,y1,z1);
  double t2 = zdepth+20., x2, y2, z2;
  line(t2,parFit,x2,y2,z2);
  TVector3 xf(x2-x1, y2-y1, z2-z1);
  TVector3 xs = itmin->fX;
  double theta = xf.Angle(xs);

  // derive recoile proton energy from theta(lab)
  // -- non-relativistic
  //	double ek = 250.*cos(theta)*cos(theta);
  //	beta = sqrt(ek*ek + 2.*ek*939.)/(ek+939.);

  // -- relativistic
  double mu = 931.494;
  double mp = 7.289 + mu;
  double mn = 8.071 + mu;

  CParticle pin(mp), nin(mn), pout(mp), nout(mn);
  pin.k(ein-sn);
  nin.k(0);

  static CKinema kinema;
  kinema.Scatter(theta, 0., pin, nin, nout, pout);

  double tmin = 0;
  int count = 0;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    double tsum = -1.;
    //先頭でhitしてから4層通過分程度時間がかかったeventは切ってしまう
    if(it == itmin){
      tsum = 0;
    }else if(it->fT - itmin->fT < 0.8){
      double xmin, ymin, zmin;
      line(it->fX.Z(), parFit, xmin, ymin, zmin);
      TVector3 Xmin(xmin, ymin, zmin);      
      double distance = (Xmin - itmin->fX).Mag();

      CParticle ntemp = nout;
      tsum  = CKinema::BetheBloch::TimeOut
	("p", "Scint", ntemp, distance/2., 300.);//plasticは2cmしかない
      if(tsum >= 0.){
	//ほんとは2cm+2cm+2cm+....と交互に時間を足していかなければならない	
	tsum += (distance/2.)/((nout.beta()+ntemp.beta())*29.979/2.);
      }
    }
    if(tsum >= 0.){
      //補正した結果、改善する方向に向かわない場合は切る
      if(abs((it->fT - tsum) - itmin->fT) < treso*2){
	tmin += it->fT - tsum;
	++count;
      }
    }
    ++it;
  }
  tmin /= count;

  Double_t x,y,z;
  line(itmin->fX.Z(),parFit,x,y,z);

  expdata->fX.SetX(x);
  expdata->fX.SetY(y);
  expdata->fX.SetZ(z);
  expdata->fX.SetT(tmin);

  expdata->CalMomentum();

#ifdef VISUALIZE
  graph->SetMarkerStyle(8);
  graph->SetMarkerSize(1);
  graph->GetXaxis()->SetLimits(-length/2., length/2.);
  graph->GetYaxis()->SetLimits(-length/2., length/2.);
  graph->SetMinimum(zdepth);
  graph->SetMaximum(zdepth+20);

  if(0){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }

  DrawLine(parFit, kRed);

  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");
  canvas->Update();

  EventLoop();
#endif

  expdata->fEventFlag = 1;

  return;
}

void EventLoop()
{
  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");

  canvas->HandleInput(kNoEvent, 0, 0);
  while (!gSystem->ProcessEvents() && canvas) {
    canvas = (TCanvas*)gROOT->GetSelectedPad();
    Int_t event = canvas->GetEvent();
    if(event == kKeyPress) break;
    if(gSystem->ProcessEvents()) break;
    gSystem->Sleep(100);
  }
}

void line(double t, double *p, double &x, double &y, double &z)
{ 
  x = p[0] + p[1]*t; 
  y = p[2] + p[3]*t;
  z = t; 
} 

//3次元における誤差を考慮した距離
double distance2(double x,double y,double z, double ex, double ey, double ez, double *p)
{ 
  TVector3 xp(x,y,z); 
  TVector3 x0(p[0], p[2], 0.); 
  TVector3 v0(p[1], p[3], 1.); 
  Double_t k = v0.Dot(xp-x0)/v0.Mag2();

  TVector3 v1 = (x0-xp)+k*v0;//x,y,zと直線を結ぶ、直線に対して垂直なベクトル

  TVector3 v2(v1.x()/ex, v1.y()/ey, v1.z()/ez);
  return v2.Mag2();
}

//  2次元(x,y)のみで距離を定義、こっちのほうがなぜか良い結果になる。
//double distance2(double x,double y,double z, double ex, double ey, double /*ez*/, double *p)
//{ 
//  double t = z;
//  double xt, yt, zt;
//  line(t, p, xt, yt, zt);
//  double d2 = pow((xt-x)/ex, 2.) + pow((yt-y)/ey, 2.);
//  return d2; 
//}

//3次元直線との距離
// double distance2(double x,double y,double z, double ex, double ey, double ez, double *p)
// { 
//   ROOT::Math::XYZVector xp(x,y,z); 
//   ROOT::Math::XYZVector x0(p[0], p[2], 0. ); 
//   ROOT::Math::XYZVector x1(p[0] + p[1], p[2] + p[3], 1. ); 
//   ROOT::Math::XYZVector u = (x1-x0).Unit(); 
//   double d2 = ((xp-x0).Cross(u)) .Mag2(); 
//   return d2; 
// }

void SumDistance2(int &, double *, double & sum, double * par, int )
{ 
  static bool first = true; 

  TGraph2DErrors* gr = dynamic_cast<TGraph2DErrors*>((TVirtualFitter::GetFitter())->GetObjectFit());
  assert(gr != 0);
  double* x = gr->GetX();
  double* y = gr->GetY();
  double* z = gr->GetZ();
  double* ex = gr->GetEX();
  double* ey = gr->GetEY();
  double* ez = gr->GetEZ();
  int npoints = gr->GetN();
  sum = 0;
  for (int i  = 0; i < npoints; ++i) { 
    double d = distance2(x[i],y[i],z[i], ex[i], ey[i], ez[i], par); 
    sum += d;
  }
  if(first) std::cout << "Total sum2 = " << sum << std::endl;

  first = false;
}

double line3Dfit(TGraph2DErrors* gr, double parFit[4])
{
  TVirtualFitter *min = TVirtualFitter::Fitter(0,4);
  min->SetObjectFit(gr);
  min->SetFCN(SumDistance2);
   
  Double_t arglist[10];
  arglist[0] = -1;
  min->ExecuteCommand("SET PRINT",arglist,1);
  
  double pStart[4] = {1,1,1,1};
  min->SetParameter(0,"x0",pStart[0],0.01,0,0);
  min->SetParameter(1,"Ax",pStart[1],0.01,0,0);
  min->SetParameter(2,"y0",pStart[2],0.01,0,0);
  min->SetParameter(3,"Ay",pStart[3],0.01,0,0);
    
  arglist[0] = 1000; // number of function calls 
  arglist[1] = 0.001; // tolerance 
  min->ExecuteCommand("MIGRAD",arglist,2);

  //if (minos) min->ExecuteCommand("MINOS",arglist,0);
  int nvpar,nparx; 
  double amin,edm, errdef;
  min->GetStats(amin,edm,errdef,nvpar,nparx);

  //  min->PrintResults(1,amin);
  //  gr->Draw("p0");

  // get fit parameters
  for (int i = 0; i <4; ++i) parFit[i] = min->GetParameter(i);

  return amin;
}

void DrawLine(double parFit[4], Color_t color)
{
  int n = 1000;
  double t0 = zdepth;
  double dt = zdepth+20;
  //  double t0 = -50;
  //  double dt = 100;
  TPolyLine3D *l = new TPolyLine3D(n);
  for (int i = 0; i <n;++i) {
    double t = t0+ dt*i/n;
    double x,y,z;
    line(t,parFit,x,y,z);
    l->SetPoint(i,x,y,z);
  }

#ifdef VISUALIZE
  l->SetLineColor(color);
  l->Draw("same");
#endif
}

void Simple(CDataHime* expdata)
{
  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(it->fE < thr){
      it = expdata->fDequeOfOneData.erase(it);
    }else{
      ++it;
    }
  }

  if(expdata->fDequeOfOneData.empty()){
    expdata->fEventFlag = 0;
    return;
  }

  std::deque<CDataHime::OneData>::iterator itmin;
  itmin = expdata->fDequeOfOneData.begin();
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  expdata->fX.SetX(itmin->fX.X());
  expdata->fX.SetY(itmin->fX.Y());
  expdata->fX.SetZ(itmin->fX.Z());
  expdata->fX.SetT(itmin->fT);

  expdata->CalMomentum();

  expdata->fEventFlag = 1;

  return;
}

void Niren(CDataHime* expdata)
{
  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(it->fE < thr){
      it = expdata->fDequeOfOneData.erase(it);
    }else{
      ++it;
    }
  }

  if(expdata->fDequeOfOneData.empty()){
    expdata->fEventFlag = 0;
    return;
  }

  std::deque<CDataHime::OneData>::iterator itmin;
  itmin = expdata->fDequeOfOneData.begin();
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  //  std::deque<CDataHime::OneData>::iterator itmin2;
  //  itmin2 = expdata->fDequeOfOneData.begin();
  //  it = expdata->fDequeOfOneData.begin();
  //  while(it != expdata->fDequeOfOneData.end()){
  //    if(it == itmin){
  //      ++it;
  //      continue;
  //    }
  //    if(itmin2->fX.Z() > it->fX.Z()) itmin2 = it;
  //    ++it;
  //  }

  Double_t zmin = itmin->fX.Z() - zdepth+1;
  Int_t idmin = zmin/2./depth+0.1;

  std::deque<CDataHime::OneData>::iterator itmin2;
  itmin2 = expdata->fDequeOfOneData.begin();
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    Double_t z = it->fX.Z() - (zdepth+1);
    Int_t id = z/2./depth+0.1;
    if(idmin+1 == id){
      itmin2 = it;
      break;
    }
    ++it;
  }
  
  if(itmin2 == expdata->fDequeOfOneData.begin()){
    expdata->fEventFlag = 0;
    return;
  }
  
  if(idmin%2){
    expdata->fX.SetX(itmin->fX.X());
    expdata->fX.SetY(itmin2->fX.Y());
  }else{
    expdata->fX.SetX(itmin2->fX.X());
    expdata->fX.SetY(itmin->fX.Y());
  }

  //  if(idmin%2){
  //    expdata->fX.SetX(itmin2->fX.X());
  //    expdata->fX.SetY(itmin->fX.Y());
  //  }else{
  //    expdata->fX.SetX(itmin->fX.X());
  //    expdata->fX.SetY(itmin2->fX.Y());
  //  }

  expdata->fX.SetZ(itmin->fX.Z());
  expdata->fX.SetT(itmin->fT);

  expdata->CalMomentum();

  expdata->fEventFlag = 1;

  return;
}

Double_t MeVtoMeVee(Double_t Tin, Double_t Tout, std::string name)
{
  Double_t a1, a2, a3, a4;
  Double_t Tein, Teout, dTe;

  if(name == "e-" || name == "e+"){
    a1 = 1;
    a2 = 0;
    a3 = 0;
    a4 = 0;
  }else if(name == "proton" || name == "deuteron" || name == "triton"){
    //NE-102(CECIL et al.)
    a1 = 0.95;
    a2 = 8.0;
    a3 = 0.1;
    a4 = 0.9;
  }else if(name == "alpha" || name == "He3"){
    //plastic(CECIL et al.)
    a1 = 0.41;
    a2 = 5.9;
    a3 = 0.065;
    a4 = 1.01;
  }else{
    //used in DEMONS
    a1 = 0.29;
    a2 = 37.492;
    a3 = 0.01;
    a4 = 0.874;
  }

  Tein = a1*Tin-a2*(1-exp(-a3*pow(Tin, a4)));
  Teout = a1*Tout-a2*(1-exp(-a3*pow(Tout, a4)));
  dTe = Tein - Teout;

  if(dTe < 0) dTe = 0;

  return dTe;
}
