   
#include <TMath.h>
#include <TGraph2D.h>
#include <TRandom2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TF2.h>
#include <TH1.h>
#include <TVirtualFitter.h>
#include <TPolyLine3D.h>
#include <Math/Vector3D.h>

#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH2D.h"
#include "TH1F.h"
#include "TH3F.h"
#include "TGraph2D.h"
#include "TGraph2DErrors.h"
#include "TSystem.h"
#include "Buttons.h"

#include <iostream>
#include <cmath>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"

void Niren(std::string filename);
void SimToExp(CDataHimeSim* data, CDataHime* expdata);
void Analysis(CDataHime* expdata);

//#define VISUALIZE 1

static const Double_t thr(3.0);
//static const Double_t treso = 0.13;
static const Double_t treso = 0.20;
//static const Double_t treso = 0.40;
static const Double_t length = 100.;
static const Double_t efflength = 36.;
static const Double_t width = 4.;
static const Double_t depth = 2.;
static const Double_t vl = 15.;
static const Double_t zoffset = 701.;

void Niren(std::string filename)
{
  TCanvas* canvas = new TCanvas("canvas","",200,10,700,700);
  std::deque<TVector3> momentum;

  {
    TFile* file = TFile::Open(filename.c_str(), "READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Koyomi",tree);
    tree->Print();

    TFile* workfile = TFile::Open("root/neutron_analysis.root","recreate");
    if(!workfile) return;

    TTree* newtree = new TTree("Hitagi", "title");

    CDataHimeSim* data = 0;
    TBranch *bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata) == TTree::kMissingBranch) return;

    CDataHime* expdata = new CDataHime;
    //     newtree->Branch("hime", &expdata, 32000, 0);
    newtree->Branch("hime", &expdata);

    Long64_t nentries = tree->GetEntriesFast();
    //    nentries = 100000;
    for(Long64_t i=0; i<nentries; i++) {
      std::cout << "\r event:" << i << std::flush;

      expdata->Reset();

      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      SimToExp(data, expdata);
      Analysis(expdata);

      if(!expdata->fEventFlag) continue;

      momentum.push_back(data->fMomentum);
      newtree->Fill();
    }
    std::cout << std::endl;

    tree->ResetBranchAddresses();
    
    workfile->Write();
    workfile->Close();
    file->Close();
  }

  {
    TFile* file = TFile::Open("root/neutron_analysis.root","READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Hitagi",tree);
    tree->Print();

    CDataHime* data = 0;
    TBranch* bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata) == TTree::kMissingBranch) return;

    TH1D* hpa = new TH1D("hpa", "title", 100, 0.-10, 0.+10);
    TH1D* hpx = new TH1D("hpx", "title", 100, 0.-10, 0.+10);
    TH1D* hpy = new TH1D("hpy", "title", 100, 0.-10, 0.+10);
    TH1D* hpz = new TH1D("hpz", "title", 100, 0.-10, 0.+10);

    Long64_t nentries = tree->GetEntriesFast();
    std::deque<TVector3>::iterator it = momentum.begin();
    for(Long64_t i=0; i<nentries; i++) {
      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      if(!data->fEventFlag){
	++it;
	continue;
      }

      hpa->Fill(data->fP.P()-it->Mag());
      hpx->Fill(data->fP.Px()+it->Px());
      hpy->Fill(data->fP.Py()+it->Py());
      hpz->Fill(data->fP.Pz()-it->Pz());
      ++it;
    }

    canvas->Divide(2,2);
    canvas->cd(1);
    hpa->Draw();
    hpa->Fit("gaus");
    canvas->cd(2);
    hpx->Draw();
    hpx->Fit("gaus");
    canvas->cd(3);
    hpy->Draw();
    hpy->Fit("gaus");
    canvas->cd(4);
    hpz->Draw();
    hpz->Fit("gaus");
    canvas->Update();

    tree->ResetBranchAddresses();
  }
}

void SimToExp(CDataHimeSim* simdata, CDataHime* expdata)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.-itsim->fPosition.X();
    Double_t xr = itsim->fPosition.X()+length/2.;
    Double_t yl = length/2.-itsim->fPosition.Y();
    Double_t yr = itsim->fPosition.Y()+length/2.;

    Double_t tl, tr, al, ar;
    if(id[0]%2){
      tl = itsim->fTime + yl/vl;
      tr = itsim->fTime + yr/vl;
      al = itsim->fEdep*exp(-yl/length);
      ar = itsim->fEdep*exp(-yr/length);
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      al = itsim->fEdep*exp(-xl/length);
      ar = itsim->fEdep*exp(-xr/length);
    }

    CDataHime::OneData* onedata = expdata->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      expdata->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    it->fTL += gRandom->Gaus(0,treso);  
    it->fTR += gRandom->Gaus(0,treso);  

    it->fT = (it->fTL+it->fTR-length/vl)/2;
    it->fE = sqrt(it->fAL*it->fAR);

    Double_t x = efflength/2.-it->fId[1]*width - width/2.;
    //    Double_t x = efflength/2.-it->fId[1]*width - width/2. + gRandom->Uniform(-width/2., width/2.);
    Double_t y = vl*(it->fTL-it->fTR)/2.;
    Double_t z = it->fId[0]*2.*depth+zoffset;
    //    Double_t z = it->fId[0]*2.*depth+zoffset + gRandom->Uniform(-depth/2., depth/2.);
    if(it->fId[0]%2){
      it->fX.SetX(x);
      it->fX.SetY(y);
    }else{
      it->fX.SetX(y);
      it->fX.SetY(x);
    }
    it->fX.SetZ(z);
    ++it;
  }
}

void Analysis(CDataHime* expdata)
{
  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(it->fE < thr){
      it = expdata->fDequeOfOneData.erase(it);
    }else{
      ++it;
    }
  }

  if(expdata->fDequeOfOneData.empty()){
    expdata->fEventFlag = 0;
    return;
  }

  std::deque<CDataHime::OneData>::iterator itmin;
  itmin = expdata->fDequeOfOneData.begin();
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

//  std::deque<CDataHime::OneData>::iterator itmin2;
//  itmin2 = expdata->fDequeOfOneData.begin();
//  it = expdata->fDequeOfOneData.begin();
//  while(it != expdata->fDequeOfOneData.end()){
//    if(it == itmin){
//      ++it;
//      continue;
//    }
//    if(itmin2->fX.Z() > it->fX.Z()) itmin2 = it;
//    ++it;
//  }

  Double_t zmin = itmin->fX.Z() - zoffset;
  Int_t idmin = zmin/2./depth+0.1;

  std::deque<CDataHime::OneData>::iterator itmin2;
  itmin2 = expdata->fDequeOfOneData.begin();
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    Double_t z = it->fX.Z() - zoffset;
    Int_t id = z/2./depth+0.1;
    if(idmin+1 == id){
      itmin2 = it;
      break;
    }
    ++it;
  }
  
  if(itmin2 == expdata->fDequeOfOneData.begin()){
    expdata->fEventFlag = 0;
    return;
  }
  
  if(idmin%2){
    expdata->fX.SetX(itmin->fX.X());
    expdata->fX.SetY(itmin2->fX.Y());
  }else{
    expdata->fX.SetX(itmin2->fX.X());
    expdata->fX.SetY(itmin->fX.Y());
  }

//  if(idmin%2){
//    expdata->fX.SetX(itmin2->fX.X());
//    expdata->fX.SetY(itmin->fX.Y());
//  }else{
//    expdata->fX.SetX(itmin->fX.X());
//    expdata->fX.SetY(itmin2->fX.Y());
//  }

  expdata->fX.SetZ(itmin->fX.Z());
  expdata->fX.SetT(itmin->fT);

  expdata->CalMomentum();

  expdata->fEventFlag = 1;

  return;
}
