
#include "TROOT.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TGraph2DErrors.h"
#include "TVector3.h"
#include "TF1.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

#include <iostream>
#include <string>
#include <cmath>
#include <deque>
#include <fstream>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "FFunction.hh"
#include "CMapFunction1D.hh"
#include "CFold.hh"
#include "CAnalysis.hh"
#include "CXmlManager.hh"

//#define VISUALIZE 1

//memory leakのerror処理はgdbを走らせれば分かるのでやってない。

void AnalysisLi(std::string filename,
		std::string facility, std::string detector,
		std::string fragment, std::string target,
		std::string beam, std::string method, Long64_t nentries = -1);

int ReadParamaters(std::string facility, std::string detector,
		   std::string fragment, std::string target);

//calculate simlation data to experiment data
void SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method);

//analysis method for experiment data
void Track(CDataHime* dataexp);
void Simple(CDataHime* dataexp);
void Niren(CDataHime* dataexp);//not optimized

int GetFirstProtonInformation(CDataHimeSim* data, CParticle& proton);
double GetTotalEnergyLoss(CDataHime* data);
void FoldChargedParticle(TVector3 momentum, CParticle& particle,
			 TVector3 dpoverp, double dele, double estrag, double multi_theta);

#include "Analysis_sub.C"

void AnalysisLi(std::string filename,
		std::string facility, std::string detector,
		std::string fragment, std::string target,
		std::string beam, std::string method, Long64_t nentries)
{
  if(!ReadParamaters(facility, detector, fragment, target)){
    std::cerr << "error occur during reading paramaters" << std::endl;
    return;
  }

  TDirectory* topdir = gDirectory;

  TCanvas* canvas = new TCanvas("canvas","",200,10,900,900);
  //  TCanvas* canvas = new TCanvas("canvas","",200,10,1200,900);

  new TH1D("rms", "title", 100, 0., 20.);
  new TH1D("theta_exp", "title", 100, 0., 3.14159);
  new TH1D("theta_cm_exp", "title", 100, 0., 3.14159);

  TH1D* htheta_lab = new TH1D("theta_lab", "title", 100, -2., 2);
  TH1D* hphi_lab = new TH1D("phi_lab", "title", 100, -2., 2);

  TH2D* hthetaenergy_lab
    = new TH2D("thetaenergy_lab", "title", 100, 0., 3.14159, 100, 0., ein+20);
  TH2D* hthetaenergy_lab_gate
    = new TH2D("thetaenergy_lab_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);
  TH2D* hthetaenergy_cm
    = new TH2D("thetaenergy_cm", "title", 100, 0., 3.14159, 100, 0., ein+20);
  TH2D* hthetaenergy_cm_gate
    = new TH2D("thetaenergy_cm_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);

  TH2D* hdee_lab
    = new TH2D("dee_lab", "title", 100, 0., 100, 100, 0., ein+20);
  TH2D* hthetade_lab
    = new TH2D("thetade_lab", "title", 100, 0., 3.14159, 100, 0., 100);
  TH3D* hthetadee_lab
    = new TH3D("thetadee_lab", "title",
	       100, 0., 3.14159,
	       100, 0., 100,
	       100, 0., ein+20);

  TH1D* htheta_cm = new TH1D("theta_cm", "title", 100, 0., 3.14159);
  TH1D* htheta_cm_gate = new TH1D("theta_cm_gate", "title", 100, 0., 3.14159);

  TH1D* hpx = new TH1D("n_px", "title", 100, 0.-2, 0.+2);
  TH1D* hpy = new TH1D("n_py", "title", 100, 0.-2, 0.+2);
  TH1D* hpz = new TH1D("n_pz", "title", 100, 0.-2, 0.+2);
  TH1D* hcpx = new TH1D("cp_px", "title", 100, 0.-2, 0.+2);
  TH1D* hcpy = new TH1D("cp_py", "title", 100, 0.-2, 0.+2);
  TH1D* hcpz = new TH1D("cp_pz", "title", 100, 0.-2, 0.+2);
  TH1D* herel = new TH1D("Erel", "title", 50, 0., 1.);

  CParticle proton = CParticleDefinition::GetParticle("proton");
  CParticle neutron = CParticleDefinition::GetParticle("neutron");
  CParticle chargedparticle = CParticleDefinition::GetParticle("9Li");
  //    CParticle chargedparticle = CParticleDefinition::GetParticle("10Be");

  {
    TFile* file = TFile::Open(filename.c_str(), "READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Koyomi",tree);

    TFile* workfile = TFile::Open("root/temp.root","recreate");
    if(!workfile) return;

    TTree* newtree = new TTree("Hitagi", "title");

    CDataHimeSim* datasim = 0;
    TBranch *bdatasim = 0;
    if(tree->SetBranchAddress("hime",&datasim,&bdatasim) == TTree::kMissingBranch) return;

    CDataHime* dataexp = new CDataHime;
    newtree->Branch("hime", &dataexp);

    topdir->cd();

    Long64_t count = 0;
    if(nentries < 0) nentries = tree->GetEntriesFast();
    for(int i=0; i<nentries; i++) {
      ++count;
      if(count%10000 == 0) std::cout << "\r converted events:" << count << std::flush;

      Long64_t entry = tree->LoadTree(i);
      bdatasim->GetEntry(entry);

      if(!datasim->fEventFlag) continue;

      dataexp->Reset();
      SimToExp(datasim, dataexp, method);

      newtree->Fill();
    }
    std::cout << "\r converted events:" << count << std::endl;

    tree->ResetBranchAddresses();
    
    workfile->Write();
    workfile->Close();
    file->Close();
  }

  {
    TFile* filesim = TFile::Open(filename.c_str(), "READ");
    if(!filesim) return;

    TTree* treesim;
    filesim->GetObject("Koyomi",treesim);

    CDataHimeSim* datasim = 0;
    TBranch *bdatasim = 0;
    if(treesim->SetBranchAddress("hime",&datasim,&bdatasim) == TTree::kMissingBranch) return;

    TFile* fileexp = TFile::Open("root/temp.root","READ");
    if(!fileexp) return;

    TTree* treeexp;
    fileexp->GetObject("Hitagi",treeexp);

    CDataHime* dataexp = 0;
    TBranch *bdataexp = 0;
    if(treeexp->SetBranchAddress("hime",&dataexp,&bdataexp) == TTree::kMissingBranch) return;

    Long64_t count = 0;
    if(nentries < 0) nentries = treesim->GetEntriesFast();
    for(Long64_t i=0, j=0; i<nentries; i++) {
      Long64_t entrysim = treesim->LoadTree(i);
      bdatasim->GetEntry(entrysim);

      if(!datasim->fEventFlag) continue;

      Long64_t entryexp = treeexp->LoadTree(j++);
      bdataexp->GetEntry(entryexp);

      //protonの真の角度分布をグラフ化してそれと比較する
      Double_t theta_lab, phi_lab, energy_lab;
      Double_t theta_cm, energy_cm;
      if(GetFirstProtonInformation(datasim, proton) != 0){
	theta_lab = proton.theta();
	phi_lab = proton.phi();
	energy_lab = proton.k();

	neutron.p(datasim->fMomentum.x(),datasim->fMomentum.y(),datasim->fMomentum.z());
	Double_t beta = neutron.p()/(neutron.e()+proton.m());
	CKinema::LorentzBoost(proton, beta, 2);

	theta_cm = proton.theta();
	energy_cm = proton.k();

	htheta_cm->Fill(theta_cm);
	hthetaenergy_lab->Fill(theta_lab, energy_lab);
	hthetaenergy_cm->Fill(theta_cm, energy_cm);
	if(dataexp->fEventFlag){
	  htheta_lab->Fill(theta_lab-dataexp->fPrp.Theta());
	  hphi_lab->Fill(phi_lab-dataexp->fPrp.Phi());

	  htheta_cm_gate->Fill(theta_cm);
	  hthetaenergy_lab_gate->Fill(theta_lab, energy_lab);
	  hthetaenergy_cm_gate->Fill(theta_cm, energy_cm);

	  hdee_lab->Fill(GetTotalEnergyLoss(dataexp),
			 energy_lab);
	  hthetade_lab->Fill(theta_lab,
			     GetTotalEnergyLoss(dataexp));
	  hthetadee_lab->Fill(theta_lab,
			      GetTotalEnergyLoss(dataexp),
			      energy_lab);
	}
      }

      if(dataexp->fEventFlag){
	++count;
	if(count%100 == 0) std::cout << "\r analysed events:" << count << std::flush;
      }else{
	continue;
      }

      TVector3 np = datasim->fMomentum;
      TVector3 cpp = datasim->fMomentumChargedParticle;

      //fold momentum
      neutron.p(dataexp->fP.Px(), dataexp->fP.Py(), dataexp->fP.Pz());
      FoldChargedParticle(cpp, chargedparticle,
			  TVector3(dpxoverp, dpyoverp, dpzoverp),
			  dele, estrag, multi_theta);

      double erel = CKinema::InvariantMass(neutron, chargedparticle)
	- neutron.m() - chargedparticle.m();
      herel->Fill(erel);

      hpx->Fill(100*(neutron.p(0)-np.Px())/np.Mag());
      hpy->Fill(100*(neutron.p(1)-np.Py())/np.Mag());
      hpz->Fill(100*(neutron.p(2)-np.Pz())/np.Mag());

      hcpx->Fill(100*(chargedparticle.p(0)-cpp.Px())/cpp.Mag());
      hcpy->Fill(100*(chargedparticle.p(1)-cpp.Py())/cpp.Mag());
      hcpz->Fill(100*(chargedparticle.p(2)-cpp.Pz())/cpp.Mag());
    }
    std::cout << "\r analysed events:" << count << std::endl;

    treeexp->ResetBranchAddresses();
    treesim->ResetBranchAddresses();
    
    fileexp->Close();
    filesim->Close();
    topdir->cd();
  }

  {
    canvas->Divide(3,3);
    //    canvas->Divide(4,3);
    canvas->cd(1);
    hpx->Draw();
    hpx->Fit("gaus");
    canvas->cd(2);
    hpy->Draw();
    hpy->Fit("gaus");
    canvas->cd(3);
    hpz->Draw();
    hpz->Fit("gaus");
    canvas->cd(4);
    hcpx->Draw();
    hcpx->Fit("gaus");
    canvas->cd(5);
    hcpy->Draw();
    hcpy->Fit("gaus");
    canvas->cd(6);
    hcpz->Draw();
    hcpz->Fit("gaus");

    canvas->cd(7);
    herel->Draw();

    if(beam == "11Li"){
      CMapFunction1D* mapf = new CMapFunction1D;
      mapf->SetFile("txt/map.txt");
      TF1* f2m1p2p = new TF1("f2m1p2p",
			     mapf,
			     static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			     (&CMapFunction1D::VRBW2MF1D),
			     0.01, 1.0, 12);
      CFold* fold = new CFold(f2m1p2p);
      TF1* ff2m1p2p = fold->GetFoldedFunction();
      ff2m1p2p->SetParameter(0, 3.5e8);
      ff2m1p2p->SetParameter(1, -12.7);
      ff2m1p2p->FixParameter(2, 3.0);
      ff2m1p2p->FixParameter(3, 0.325/2.);
      ff2m1p2p->FixParameter(4, neutron.m());
      ff2m1p2p->FixParameter(5, chargedparticle.m());
      ff2m1p2p->SetParameter(6, 1.96e-6);
      ff2m1p2p->SetParameter(7, 0.22);
      ff2m1p2p->SetParameter(8, 0.09);
      ff2m1p2p->SetParLimits(8, 0.05,0.2);
      ff2m1p2p->SetParameter(9, 7.27e-6);
      ff2m1p2p->SetParameter(10, 0.53);
      ff2m1p2p->SetParameter(11, 0.45);
      ff2m1p2p->SetParLimits(11, 0.3,6);
      ff2m1p2p->Draw("same");
      TFitResultPtr frpf2m1p2p = herel->Fit(ff2m1p2p, "Sb", "", 0.01, 0.8);

      TF1* f2m = new TF1("f2m",
			 mapf,
			 static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			 (&CMapFunction1D::VRMF1D),
			 0.01, 1.0, 6);
      CFold* foldf2m = new CFold(f2m);
      TF1* ff2m = foldf2m->GetFoldedFunction();
      ff2m->SetParameter(0, frpf2m1p2p->Parameter(0));
      ff2m->SetParameter(1, frpf2m1p2p->Parameter(1));
      ff2m->SetParameter(2, frpf2m1p2p->Parameter(2));
      ff2m->SetParameter(3, frpf2m1p2p->Parameter(3));
      ff2m->SetParameter(4, frpf2m1p2p->Parameter(4));
      ff2m->SetParameter(5, frpf2m1p2p->Parameter(5));
      ff2m->SetLineStyle(2);
      ff2m->DrawCopy("same");

      TF1* f1p = new TF1("f1p",
			 mapf,
			 static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			 (&CMapFunction1D::BWMF1D),
			 0.01, 1.0, 6);
      CFold* foldf1p = new CFold(f1p);
      TF1* ff1p = foldf1p->GetFoldedFunction();
      ff1p->SetParameter(0, frpf2m1p2p->Parameter(0)*frpf2m1p2p->Parameter(6));
      ff1p->SetParameter(1, frpf2m1p2p->Parameter(7));
      ff1p->SetParameter(2, frpf2m1p2p->Parameter(8));
      ff1p->SetLineStyle(3);
      ff1p->DrawCopy("same");

      TF1* f2p = new TF1("f2p",
			 mapf,
			 static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			 (&CMapFunction1D::BWMF1D),
			 0.01, 1.0, 6);
      CFold* foldf2p = new CFold(f2p);
      TF1* ff2p = foldf2p->GetFoldedFunction();
      ff2p->SetParameter(0, frpf2m1p2p->Parameter(0)*frpf2m1p2p->Parameter(9));
      ff2p->SetParameter(1, frpf2m1p2p->Parameter(10));
      ff2p->SetParameter(2, frpf2m1p2p->Parameter(11));
      ff2p->SetLineStyle(4);
      ff2p->DrawCopy("same");
    }

    canvas->cd(8);
    htheta_lab->Draw();
    htheta_lab->Fit("gaus", "", "same");

    canvas->cd(9);
    hphi_lab->Draw();
    hphi_lab->Fit("gaus", "", "same");

    canvas->Update();
  }
}

