#ifndef ANALYZENEUT_HH
#define ANALYZENEUT_HH

class CAnalyzeNeut
{
public:
  CAnalyzeNeut(std::string filename);
  ~CAnalyzeNeut();
  
  void Main(std::string facility, std::string detector,
	    std::string fragment, std::string target,
	    std::string beam, std::string method, Long64_t nentries);

  int ReadParamaters(std::string facility, std::string detector,
		     std::string fragment, std::string target);
  int CreateHist();
  int CreateExpData(std::string method, Long64_t nentries);
  int AnalysisExpData(Long64_t nentries);
  int DrawHist(std::string beam);

private:
  //calculate simlation data to experiment data
  void SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method);

  //analysis method for experiment data
  void Track(CDataHime* dataexp);
  void Simple(CDataHime* dataexp);
  void Niren(CDataHime* dataexp);//not optimized

  int GetFirstProtonInformation(CDataHimeSim* data, CParticle& proton);
  double GetTotalEnergyLoss(CDataHime* data);
  void FoldChargedParticle(TVector3 momentum, CParticle& particle,
			   TVector3 dpoverp, double dele, double estrag, double multi_theta);

private:
  TCanvas* canvas;

  TDirectory* topdir;
  TFile *filesim, *fileexp;

  CParticle proton;
  CParticle neutron;
  CParticle chargedparticle;

  //hist
  TH1D *hamin;
  TH1D *htheta_exp, *htheta_cm_exp;

  TH1D *htheta_lab, *hphi_lab;

  TH2D *hthetaenergy_lab, *hthetaenergy_lab_gate;
  TH2D *hthetaenergy_cm, *hthetaenergy_cm_gate;

  TH2D *hdee_lab, *hthetade_lab;
  TH3D *hthetadee_lab;

  TH1D *htheta_cm, *htheta_cm_gate;

  TH1D *hpx, *hpy, *hpz;
  TH1D *hcpx, *hcpy, *hcpz;
  TH1D *herel;

  //parameters
  Double_t dpxoverp, dpyoverp, dpzoverp, dele, estrag, multi_theta;
  Int_t thrmethod;
  Double_t thr, treso, attlength, vl;
  Int_t rotation;
  Double_t length, vlength, width, depth, gap, zdepth, zdepth_back, shift;
  Int_t multihit;
  Double_t aminmin, ein, sp;
  Double_t step;
};

#endif
