   
#include <TMath.h>
#include <TGraph2D.h>
#include <TRandom2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TF2.h>
#include <TH1.h>
#include <TVirtualFitter.h>
#include <TPolyLine3D.h>
#include <Math/Vector3D.h>

void line(double t, double *p, double &x, double &y, double &z)
{ 
  x = p[0] + p[1]*t; 
  y = p[2] + p[3]*t;
  z = t; 
} 

double distance2(double x,double y,double z, double *p)
{ 
  ROOT::Math::XYZVector xp(x,y,z); 
  ROOT::Math::XYZVector x0(p[0], p[2], 0. ); 
  ROOT::Math::XYZVector x1(p[0] + p[1], p[2] + p[3], 1. ); 
  ROOT::Math::XYZVector u = (x1-x0).Unit(); 
  double d2 = ((xp-x0).Cross(u)) .Mag2(); 
  return d2; 
}

void SumDistance2(int &, double *, double & sum, double * par, int )
{ 
  static bool first = true; 

  TGraph2D * gr = dynamic_cast<TGraph2D*>( (TVirtualFitter::GetFitter())->GetObjectFit() );
  assert(gr != 0);
  double * x = gr->GetX();
  double * y = gr->GetY();
  double * z = gr->GetZ();
  int npoints = gr->GetN();
  sum = 0;
  for (int i  = 0; i < npoints; ++i) { 
    double d = distance2(x[i],y[i],z[i],par); 
    sum += d;
#ifdef DEBUG
    if (first) std::cout << "point " << i << "\t" 
			 << x[i] << "\t" 
			 << y[i] << "\t" 
			 << z[i] << "\t" 
			 << std::sqrt(d) << std::endl; 
#endif
  }
  if(first) std::cout << "Total sum2 = " << sum << std::endl;

  first = false;
}

void line3Dfit(TGraph2D* gr, double parFit[4])
{
  TVirtualFitter *min = TVirtualFitter::Fitter(0,4);
  min->SetObjectFit(gr);
  min->SetFCN(SumDistance2);
   
  Double_t arglist[10];
   arglist[0] = 3;
   min->ExecuteCommand("SET PRINT",arglist,1);
  
   double pStart[4] = {1,1,1,1};
   min->SetParameter(0,"x0",pStart[0],0.01,0,0);
   min->SetParameter(1,"Ax",pStart[1],0.01,0,0);
   min->SetParameter(2,"y0",pStart[2],0.01,0,0);
   min->SetParameter(3,"Ay",pStart[3],0.01,0,0);
    
   arglist[0] = 1000; // number of function calls 
   arglist[1] = 0.001; // tolerance 
   min->ExecuteCommand("MIGRAD",arglist,2);

  //if (minos) min->ExecuteCommand("MINOS",arglist,0);
   int nvpar,nparx; 
   double amin,edm, errdef;
   min->GetStats(amin,edm,errdef,nvpar,nparx);
   min->PrintResults(1,amin);
   gr->Draw("p0");

   // get fit parameters
   for (int i = 0; i <4; ++i) parFit[i] = min->GetParameter(i);
}

void DrawLine(double parFit[4], Color_t color)
{
  int n = 1000;
  double t0 = 0;
  double dt = 10;
  TPolyLine3D *l = new TPolyLine3D(n);
  for (int i = 0; i <n;++i) {
    double t = t0+ dt*i/n;
    double x,y,z;
    line(t,parFit,x,y,z);
    l->SetPoint(i,x,y,z);
  }
  l->SetLineColor(color);
  l->Draw("same");
}

TGraph2D* CreateGraph(double par[4], Int_t nd)
{
  TGraph2D * gr = new TGraph2D();

  for (Int_t N=0; N<nd; N++) {
    double x,y,z = 0;
    double t = gRandom->Uniform(0,10);
    line(t,par,x,y,z);

    double err = 1;
    x += gRandom->Gaus(0,err);  
    y += gRandom->Gaus(0,err);  
    z += gRandom->Gaus(0,err);  
    gr->SetPoint(N,x,y,z);
  }

  return gr;
}

void fit()
{ 
  gStyle->SetOptStat(0);
  gStyle->SetOptFit();

  double p0[4] = {10,20,1,2};
  TGraph2D* gr = CreateGraph(p0, 4);

  double parFit[4];
  line3Dfit(gr, parFit);
  DrawLine(parFit, kRed);
  //  DrawLine(p0, kBlue);
}
