
#include "TGraph.h"

#include <iostream>
#include <cmath>
#include <vector>

int MeVee();
Double_t MeVtoMeVee(Double_t Tin, Double_t Tout, std::string name);

int MeVee()
{
  TGraph* gp;
  TGraph* ga;
  TGraph* gi;
  std::vector<Double_t> v, vp, va, vi;

  for(Double_t T = 0; T < 40; T += 1){
    Double_t Tp = MeVtoMeVee(T, 0, "proton");
    Double_t Ta = MeVtoMeVee(T, 0, "alpha");
    Double_t Ti = MeVtoMeVee(T, 0, "C12[0.0]");
    v.push_back(T);
    vp.push_back(Tp);    
    va.push_back(Ta);    
    vi.push_back(Ti);    

    gp = new TGraph(v.size(), &(*v.begin()), &(*vp.begin()));
    ga = new TGraph(v.size(), &(*v.begin()), &(*va.begin()));
    gi = new TGraph(v.size(), &(*v.begin()), &(*vi.begin()));
  }

  gp->Draw("al");
  ga->Draw("lsame");
  gi->Draw("lsame");

  return 0;
}

Double_t MeVtoMeVee(Double_t Tin, Double_t Tout, std::string name)
{
  Double_t a1, a2, a3, a4;
  Double_t Tein, Teout, dTe;

  if(name == "e-" || name == "e+"){
    a1 = 1;
    a2 = 0;
    a3 = 0;
    a4 = 0;
  }else if(name == "proton" || name == "deuteron" || name == "triton"){
    //NE-102(CECIL et al.)
    a1 = 0.95;
    a2 = 8.0;
    a3 = 0.1;
    a4 = 0.9;
  }else if(name == "alpha" || name == "He3"){
    //plastic(CECIL et al.)
    a1 = 0.41;
    a2 = 5.9;
    a3 = 0.065;
    a4 = 1.01;
  }else{
    //used in DEMONS
    a1 = 0.29;
    a2 = 37.492;
    a3 = 0.01;
    a4 = 0.874;
  }

  Tein = a1*Tin-a2*(1-exp(-a3*pow(Tin, a4)));
  Teout = a1*Tout-a2*(1-exp(-a3*pow(Tout, a4)));
  dTe = Tein - Teout;

  if(dTe < 0) dTe = 0;

  return dTe;
}
