#include "TROOT.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TRandom3.h"
#include "TH1D.h"
#include "TMath.h"

#include <iostream>
#include <sstream>

#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "CKinema.hh"

void Resolution()
{
  CParticle p1 = CParticleDefinition::GetParticle("neutron");
  CParticle p2 = CParticleDefinition::GetParticle("9Li");

  //  double ein(70);
  //  double ein(200);
  //  double ein(230);
  double ein(250);

  p1.k(ein);
  p2.k(ein*9);

//  double p1_sigma[] = {p1.p()*9.3e-3,
//		       p1.p()*3.6e-3,
//		       p1.p()*8.8e-3};//lneut
//  double p1_sigma[] = {p1.p()*1.0e-2,
//		       p1.p()*1.0e-2,
//		       p1.p()*1.0e-2};//lneut
  double p1_sigma[] = {p1.p()*1.6e-3,
		       p1.p()*1.5e-3,
		       p1.p()*3.3e-3};//hime 130ps 230MeV

  //1g/cm2
  double dele(31.57);
  double estrag(1.0);
  double multi_theta(1.4e-3);

  //2g/cm2
//  double dele(63.4);
//  double estrag(1.4);
//  double multi_theta(1.9e-3);

  double detodp_coef = (p2.k()+p2.m())/(p2.k()+2*p2.m())/p2.k()*p2.p();
  //hime
  double p2_sigma[] = {p2.p()/700, p2.p()/700, p2.p()/700,
  		       detodp_coef*dele, detodp_coef*estrag, multi_theta};
  //lneut
  //  double p2_sigma[] = {4.59e-3*p2.p(), 4.53e-3*p2.p(), 3.21e-3*p2.p(),
  //  double p2_sigma[] = {2.28e-3*p2.p(), 2.14e-3*p2.p(), 3.21e-3*p2.p(),
  //  		       detodp_coef*dele, detodp_coef*estrag, multi_theta};

  TRandom3 random; 
  CKinema kinema;

  TFile* f = new TFile("root/temp.root", "recreate");
  TCanvas* c = gROOT->MakeDefCanvas();
  for(int j = 0; j < 8; ++j){
    double erel = 0.0125*pow(2,j);
    
    std::ostringstream oss;
    oss << "hist" << j;
    TH1D* h = new TH1D(oss.str().c_str(), "title", 300, 0.0, 2.0);
    for(int i = 0; i < 20000; ++i){

      CKinema::FissionErel(p1, p2, erel, ein);

//      p1.p(p1.p(0)+random.Uniform(-p1_sigma[0]/2.,p1_sigma[0]/2.),
//	   p1.p(1)+random.Uniform(-p1_sigma[1]/2.,p1_sigma[1]/2.),
//	   random.Gaus(p1.p(2), p1_sigma[2]));
      p1.p(random.Gaus(p1.p(0), p1_sigma[0]),
	   random.Gaus(p1.p(1), p1_sigma[1]),
	   random.Gaus(p1.p(2), p1_sigma[2]));

      //energy loss difference
      p2.p(p2.p(0),
	   p2.p(1),
	   p2.p(2)+random.Uniform(-p2_sigma[3]/2., p2_sigma[3]/2.));
      //multiple scattering and energy stragglling
      p2.p(random.Gaus(p2.p(0), p2_sigma[5]*p2.p(2)),
	   random.Gaus(p2.p(1), p2_sigma[5]*p2.p(2)),
	   random.Gaus(p2.p(2), p2_sigma[4]));	   
      //detector resolution
      p2.p(random.Gaus(p2.p(0), p2_sigma[0]),
	   random.Gaus(p2.p(1), p2_sigma[1]),
	   random.Gaus(p2.p(2), p2_sigma[2]));

      double erel_exp = kinema.InvariantMass(p1, p2)-p1.m()-p2.m();

      h->Fill(erel_exp);
    }

    h->Draw();
    c->Update();

    h->Write();

    delete h;
  }
  f->Write();
  delete f;
  delete c;
}
