#include "TROOT.h"
#include "TFile.h"
#include "TRandom3.h"
#include "TH1D.h"
#include "TF1.h"
#include "TMath.h"

#include "CParticle.hh"
#include "CKinema.hh"
#include "FFunction.hh"

double InvariantMassOneEvent(double erel, double beta,
			     CParticle& p1, CParticle& p2,
			     double p1_sigma[3], double p2_sigma[6]);

void Spectrum()
{
  double u(931.494);

  int a1(1), a2(9);
  double p1_me(8.0713171);
  double p2_me(24.954264);
  CParticle p1(p1_me+u*a1),p2(p2_me+u*a2);

//  double ein(200);
//  double p1_sigma[] = {0.9288, 0.8853, 1.716};//130ps

  double ein(230);
  double p1_sigma[] = {1.12, 1.02, 2.38};//130ps //200MeV //9m

  //  double ein(250);
  //  double p1_sigma[] = {1.056, 1.016, 2.258};//130ps //250MeV

  //11Li
  //2g/cm2
  double dele(0.18);
  double estrag(1.3);
  double multi_theta(1.9e-3);

  //11Be
  //2g/cm2 //250MeV
//  double dele(49.82);
//  double estrag(1.8);
//  double multi_theta(2.1e-3);

  //11B
  //1g/cm2 //250MeV
//  double dele(56.61);
//  double estrag(1.6);
//  double multi_theta(1.7e-3);

  //1.5g/cm2 //250MeV
//  double dele(85.43);
//  double estrag(2.0);
//  double multi_theta(2.1e-3);

  //2g/cm2 //250MeV
//  double dele(114.6);
//  double estrag(2.3);
//  double multi_theta(2.4e-3);

  p2.k(ein*a2);
  //  double p2_sigma[] = {p2.p()/700., p2.p()/700., p2.p()/700., dele/(ein*a2)/2*p2.p(), estrag, multi_theta};
  double p2_sigma[] = {p2.p()/700., p2.p()/700., p2.p()/700.,
		       (p2.k()+p2.m())/(p2.k()+2*p2.m())*dele/p2.k()*p2.p(),
		       estrag, multi_theta};

  CParticle pv(u);
  pv.k(ein);
  double beta = pv.beta();

  TF1* f2m = new TF1("f2m", VirtualResonance, 0, 3, 5);
  //  f2m->SetParameters(-15.4, 3.0, 0.25, p1.m(), p2.m());  //a, r0, Eb, mass1, mass2
  f2m->SetParameters(-12.7, 3.0, 0.325/2., p1.m(), p2.m());  //a, r0, Eb, mass1, mass2
  TF1* f1m = new TF1("f1m", VirtualResonance, 0, 3, 5);
  //  f1m->SetParameters(-5.6, 3.0, 0.25, p1.m(), p2.m());
  f1m->SetParameters(-5.6, 3.0, 0.325, p1.m(), p2.m());
  TF1* f1p = new TF1("f1p", BrietWigner, 0, 3, 2);
  f1p->SetParameters(0.22,0.09);
  TF1* f2p = new TF1("f2p", BrietWigner, 0, 3, 2);
  f2p->SetParameters(0.53,0.45);

  TF1* fg = new TF1("f", "gaus", 0, 3);
  fg->SetParameters(1, 1, 0.001);

  // {dpx,dpy,dpz}
  //  double p1_sigma_exp[] = {10, 10, 10};
  //  double p1_sigma_exp[] = {2.88, 2.88, 3.28};
  //  double p1_sigma_exp[] = {21./7.*2.88/sqrt(2.), 21./7.*2.88/sqrt(2.), 21./7.*3.03/sqrt(2.)};
  //  double p1_sigma_exp[] = {25./7.*2.88/sqrt(2.), 25./7.*2.88/sqrt(2.), 25./7.*3.03/sqrt(2.)};
  //  double p1_sigma_exp[] = {10./7.*2.88/sqrt(2.), 10./7.*2.88/sqrt(2.), 10./7.*3.03/sqrt(2.)};
  //  double p1_sigma_exp[] = {2.88/sqrt(2.), 2.88/sqrt(2.), 3.03/sqrt(2.)};
  //  double p1_sigma[] = {0.0, 0.0, 0.0};

  // {dpx,dpy,dpz,pdz by target}
  //  double p2_sigma_exp[] = {20, 20, 20};
  //  double p2_sigma_exp[] = {2.*5./3.*9.81, 2.*5./3.*9.81, 2.*3./3.*9.81, 0.0};
  //  double p2_sigma_exp[] = {9.81, 9.81, 9.81, 14.653};
  //  double p2_sigma_exp[] = {0.0, 0.0, 0.0};
  //  double p2_sigma[] = {0.0, 0.0, 0.0, 0.0};

  TH1D* h = new TH1D("hall", "no folding", 100, 0., 3.0);
  TH1D* h2m = new TH1D("h2m", "(0s1/2)v, Jpi=2-", 100, 0., 3.0);
  TH1D* h1m = new TH1D("h1m", "(0s1/2)v, Jpi=1-", 100, 0., 3.0);
  TH1D* h1p = new TH1D("h1p", "(0p1/2)v, Jpi=1+", 100, 0., 3.0);
  TH1D* h2p = new TH1D("h2p", "(0p1/2)v, Jpi=2+", 100, 0., 3.0);
  TH1D* hexp = new TH1D("hexp", "fold by experimenal resolution", 100, 0., 3.0);
  TH1D* hexps = new TH1D("hexps", "fold by experimenal resolution", 100, 0., 3.0);
  TH1D* hexpl = new TH1D("hexpl", "fold by experimenal resolution", 100, 0., 3.0);
//  for(int i = 0; i < 100000; ++i){
//    double reso2m = InvariantMassOneEvent(f2m->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
//    double reso1m = InvariantMassOneEvent(f1m->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
//    double reso1p = InvariantMassOneEvent(f1p->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
//    double reso2p = InvariantMassOneEvent(f2p->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
//    h->Fill(reso2m);
//    h->Fill(reso1m);
//    h->Fill(reso1p);
//    h->Fill(reso2p);
//    h2m->Fill(reso2m);
//    h1m->Fill(reso1m);
//    h1p->Fill(reso1p);
//    h2p->Fill(reso2p);
//  }

//  for(int i = 0; i < 2500; ++i){
//    double reso2m = InvariantMassOneEvent(f2m->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    double reso1m = InvariantMassOneEvent(f1m->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    double reso1p = InvariantMassOneEvent(f1p->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    double reso2p = InvariantMassOneEvent(f2p->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    hexp->Fill(reso2m);
//    hexp->Fill(reso1m);
//    hexp->Fill(reso1p);
//    hexp->Fill(reso2p);
//  }
//
//  for(int i = 0; i < 250; ++i){
//    double reso2m = InvariantMassOneEvent(f2m->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    double reso1m = InvariantMassOneEvent(f1m->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    double reso1p = InvariantMassOneEvent(f1p->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    double reso2p = InvariantMassOneEvent(f2p->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp);
//    hexps->Fill(reso2m);
//    hexps->Fill(reso1m);
//    hexps->Fill(reso1p);
//    hexps->Fill(reso2p);
//  }

  for(int i = 0; i < 40000; ++i){
    double reso2m = InvariantMassOneEvent(f2m->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
    hexpl->Fill(reso2m);
    h2m->Fill(reso2m);

    //    double reso1m = InvariantMassOneEvent(f1m->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
    //    hexpl->Fill(reso1m);
    //    h1m->Fill(reso1m);

    if(i%5 == 0){
      double reso1p = InvariantMassOneEvent(f1p->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
      hexpl->Fill(reso1p);
      h1p->Fill(reso1p);
    }
    if(i%5 == 0 or i%5 == 1 or i%5 == 2 or i%5 == 3){
      double reso2p = InvariantMassOneEvent(f2p->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma);
      hexpl->Fill(reso2p);
      h2p->Fill(reso2p);
    }
  }

  //  for(int i = 0; i < 100000; ++i){
    //    h->Fill(InvariantMassOneEvent(fg->GetRandom(), beta, p1, p2, p1_sigma, p2_sigma));
    //    hexp->Fill(InvariantMassOneEvent(fg->GetRandom(), beta, p1, p2, p1_sigma_exp, p2_sigma_exp));
  //  }

  TFile* f = new TFile("root/temp.root", "recreate");
  h->Write();
  h2m->Write();
  h1m->Write();
  h1p->Write();
  h2p->Write();
  hexp->Write();
  hexps->Write();
  hexpl->Write();
  f->Write();
  delete f;
}

double InvariantMassOneEvent(double erel, double beta,
			     CParticle& p1, CParticle& p2,
			     double p1_sigma[3], double p2_sigma[6])
{
  static TRandom3 random;
  static CKinema kinema;

  p1.k(0);
  p2.k(0);

  double theta = acos(2*random.Uniform() - 1);
  double phi = TMath::Pi()*(2*random.Uniform() - 1);
  kinema.Fission(p1, p2, erel, theta, phi);
  kinema.LorentzBoost(p1, -beta, 2);
  kinema.LorentzBoost(p2, -beta, 2);
//  p1.p(p1.p(0)+random.Uniform(-p1_sigma[0]/2., p1_sigma[0]/2.),
//       p1.p(1)+random.Uniform(-p1_sigma[1]/2., p1_sigma[1]/2.),
//       random.Gaus(p1.p(2), p1_sigma[2]));
  p1.p(random.Gaus(p1.p(0), p1_sigma[0]),
       random.Gaus(p1.p(1), p1_sigma[1]),
       random.Gaus(p1.p(2), p1_sigma[2]));

  //energy stragglling
  p2.p(p2.p(0),
       p2.p(1),
       random.Gaus(p2.p(2), p2_sigma[4]));
  //multiple scattering and energy defference
  p2.p(random.Gaus(p2.p(0), p2_sigma[5]*p2.p(2)/sqrt(2)),
       random.Gaus(p2.p(1), p2_sigma[5]*p2.p(2)/sqrt(2)),
       p2.p(2)+random.Uniform(-p2_sigma[3]/2., p2_sigma[3]/2.));
  //detector resolution
  p2.p(random.Gaus(p2.p(0), p2_sigma[0]),
       random.Gaus(p2.p(1), p2_sigma[1]),
       random.Gaus(p2.p(2), p2_sigma[2]));

  return kinema.InvariantMass(p1, p2)-p1.m()-p2.m();
}
