
#include "readin.hh"

#include <iostream>
#include <iterator>
#include <fstream>
#include <sstream>
#include <algorithm>

int FReadFile::FLessX::m_index(0);
int FReadFile::FLessY::m_index(1);

int main()
{
  std::string filename("101026-bn-3.1T-map.y.table");  

  std::vector<std::vector<double> > xy;
  FReadFile::ReadIn(filename, xy);
  FReadFile::Print(xy);

  std::vector<std::vector<double> > xy_new;
  FReadFile::ReFill(xy, xy_new, 0, 1, 2);
  FReadFile::Print(xy_new);

  return 0;
}

int FReadFile::ReadIn(std::string filename, std::vector<std::vector<double> >& xy)
{
  std::ifstream ifs(filename.c_str());
  if(!ifs.is_open()){
    std::cout << filename << " dose not exist" << std::endl;
    return 0;
  }

  while(1){
    std::string str;
    std::getline(ifs, str);
    if(ifs.eof()) break;
    if(str.find("#") != std::string::npos) continue;
    std::istringstream iss(str);

    std::vector<double> x;
    double xtemp;
    while(iss >> xtemp) x.push_back(xtemp);
    xy.push_back(x);
  }

  return 1;
}

int FReadFile::ReFill(std::vector<std::vector<double> > xy, 
		      std::vector<std::vector<double> >& xy_new, 
		      int xindex, int yindex, int findex)
{
  FLessX::SetIndex(xindex);
  std::sort(xy.begin(), xy.end(), FLessX());

  FLessY::SetIndex(yindex);
  std::stable_sort(xy.begin(), xy.end(), FLessY());

  std::vector<std::vector<double> >::iterator it = xy.begin();
  while(it != xy.end()){
    std::vector<double> x_new;
    double y = (*it)[yindex];
    while(it != xy.end() && y == (*it)[yindex]){
      x_new.push_back((*it)[findex]);
      ++it;
    }
    xy_new.push_back(x_new);
  }

#ifndef NDEBUG
  std::vector<std::vector<double> >::iterator it_new = xy_new.begin();
  if(it_new == xy_new.end()) abort();
  while(1){
    std::vector<double>::size_type a = it_new->size();
    ++it_new; if(it_new == xy_new.end()) break;
    std::vector<double>::size_type b = it_new->size();
    if(a != b){
      std::cout << "size of one line changed!! map must be rectangle"
		<< std::endl;
      std::abort();
    }
  }
#endif
}

void FReadFile::Print(const std::vector<std::vector<double> >& xy)
{
  std::vector<std::vector<double> >::const_iterator it = xy.begin();
  while(it != xy.end()){
    std::copy(it->begin(), it->end(),
	      std::ostream_iterator<double>(std::cout, " "));
    std::cout << std::endl;
    ++it;
  }
}
