
#include "CMagnet.hh"

#include "FReadFile.hh"
#include "CMapFunction2D.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"

#include "TROOT.h"
#include "TMath.h"
#include "TCanvas.h"
#include "TVector2.h"
#include "TGraph.h"
#include "TF2.h"

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>
#include <cmath>

int test_mag2()
{
  TCanvas* canvas = new TCanvas("canvas", "title", 600,600);

  std::string filename("root/101026-bn-3.1T-map.y.table");
  CMapFunction::type_map2d xy;
  FReadFile::ReadIn(filename, xy);
  CMapFunction::type_map2d xy_new;
  FReadFile::ReFillQuad(xy, xy_new, 0, 1, 2);
  //  FReadFile::ReFill(xy, xy_new, 0, 1, 2);
  FReadFile::Print(xy);

  CMapFunction2D* map2d = new CMapFunction2D;
  if(!map2d->SetFile(xy_new,-3,0.01,-3,0.01)) return -1;
  map2d->Print();
  TF1* fmap = map2d->GetMapFunction();

  std::vector<double> vx,vy;
  vx.push_back(0);
  vy.push_back(0);
  //  double theta = TMath::Pi()*3./4;
  double theta = TMath::Pi()/2;
//  double rstep = 1e-4;
//  double thetastep = 1e-6;
  double rstep = 1e-3;
  double thetastep = 1e-5;
  double rangelimit = 1e2;
  CParticle proton = CParticleDefinition::GetParticle("proton");
  proton.k(500);
  CMagnet::CalTrack(map2d, vx, vy, theta, rstep, thetastep, rangelimit,
		    proton.m(), proton.beta(), 1);

  std::vector<double> vx2,vy2;
  vx2.push_back(0);
  vy2.push_back(0);
  CMagnet::CalTrack(map2d, vx2, vy2, theta, rstep, thetastep, rangelimit,
		    2*proton.m(), proton.beta(), 1);

  TGraph* graph = new TGraph(vx.size(), &(*vx.begin()), &(*vy.begin()));
  TGraph* graph2 = new TGraph(vx2.size(), &(*vx2.begin()), &(*vy2.begin()));

  fmap->Draw("colz");
  graph->SetLineColor(1);
  graph->SetLineWidth(3);
  graph->Draw("same");
  graph2->SetLineColor(28);
  graph2->SetLineWidth(3);
  graph2->Draw("same");

  canvas->Update();

  return 0;
}
