
#include "CMagnet.hh"

#include "FReadFile.hh"
#include "CMapFunction2D.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"

#include "TROOT.h"
#include "TMath.h"
#include "TCanvas.h"
#include "TVector2.h"
#include "TGraph.h"
#include "TF2.h"

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>
#include <cmath>

int test2()
{
  TCanvas* canvas = new TCanvas("canvas", "title", 600,600);

  std::string filename("root/101026-bn-3.1T-map.y.table");
  CMapFunction::type_map2d xy;
  FReadFile::ReadIn(filename, xy);
  CMapFunction::type_map2d xy_new;
  FReadFile::ReFillQuad(xy, xy_new, 0, 1, 2);
  //  FReadFile::ReFill(xy, xy_new, 0, 1, 2);
  //  FReadFile::Print(xy);

  CMapFunction2D* map2d = new CMapFunction2D;
  if(!map2d->SetFile(xy_new,-3,0.01,-3,0.01)) return -1;
  //  map2d->Print();
  TF1* fmap = map2d->GetMapFunction();

  //  double theta = TMath::Pi()*3./4;
  double theta = TMath::Pi()*(1./6+1./2);
  //  double rstep = 1e-5;
  //  double thetastep = 1e-7;
  double rstep = 1e-4;
  double thetastep = 1e-6;
  //  double rstep = 1e-3;
  //  double thetastep = 1e-5;
//  double rstep = 1e-2;
//  double thetastep = 1e-4;
  double rangelimit = 1e2;
  CParticle proton = CParticleDefinition::GetParticle("proton");
  proton.k(250);

  double zinipos = -3;  
  double xinipos = zinipos/tan(theta);
  std::vector<double> vx,vy;
  vx.push_back(xinipos);
  vy.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx, vy, theta, rstep, thetastep, rangelimit,
		    proton.m(), proton.beta(), 1);

  std::vector<double> vx2,vy2;
  vx2.push_back(xinipos);
  vy2.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx2, vy2, theta, rstep, thetastep, rangelimit,
		    2*proton.m(), proton.beta(), 1);

  std::vector<double> vx3,vy3;
  vx3.push_back(xinipos);
  vy3.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx3, vy3, theta, rstep, thetastep, rangelimit,
		    3*proton.m(), proton.beta(), 1);

  std::vector<double> vx4,vy4;
  vx4.push_back(xinipos);
  vy4.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx4, vy4, theta, rstep, thetastep, rangelimit,
		    4*proton.m(), proton.beta(), 0);

  std::cout << "A/Z=1 " << vx.back() << " " << vy.back() << std::endl;
  std::cout << "A/Z=2 " << vx2.back() << " " << vy2.back() << std::endl;
  std::cout << "A/Z=3 " << vx3.back() << " " << vy3.back() << std::endl;
  std::cout << "q=0 "   << vx4.back() << " " << vy4.back() << std::endl;

  TGraph* graph = new TGraph(vx.size(), &(*vx.begin()), &(*vy.begin()));
  TGraph* graph2 = new TGraph(vx2.size(), &(*vx2.begin()), &(*vy2.begin()));
  TGraph* graph3 = new TGraph(vx3.size(), &(*vx3.begin()), &(*vy3.begin()));
  TGraph* graph4 = new TGraph(vx4.size(), &(*vx4.begin()), &(*vy4.begin()));

  fmap->Draw("colz");
  graph->SetLineColor(1);
  graph->SetLineWidth(3);
  graph->Draw("same");
  graph2->SetLineColor(28);
  graph2->SetLineWidth(3);
  graph2->Draw("same");
  graph3->SetLineColor(4);
  graph3->SetLineWidth(3);
  graph3->Draw("same");
  graph4->SetLineColor(5);
  graph4->SetLineWidth(3);
  graph4->Draw("same");

  canvas->Update();

  return 0;
}
