
#include "CMagnet.hh"
#include "CCoordinateTransformation.hh"

#include "FReadFile.hh"
#include "CMapFunction2D.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"

#include "TROOT.h"
#include "TMath.h"
#include "TCanvas.h"
#include "TVector2.h"
#include "TGraph.h"
#include "TF2.h"

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>
#include <cmath>

int test3()
{
  TCanvas* canvas = new TCanvas("canvas", "title", 600,600);

  std::string filename("root/101026-bn-3.1T-map.y.table");
  CMapFunction::type_map2d xy;
  FReadFile::ReadIn(filename, xy);
  CMapFunction::type_map2d xy_new;
  FReadFile::ReFillQuad(xy, xy_new, 0, 1, 2);
  //  FReadFile::ReFill(xy, xy_new, 0, 1, 2);
  //  FReadFile::Print(xy);

  CMapFunction2D* map2d = new CMapFunction2D;
  if(!map2d->SetFile(xy_new,-3,0.01,-3,0.01)) return -1;
  //  map2d->Print();
  TF1* fmap = map2d->GetMapFunction();

  CCoordinateTransformation ct(0, 0, TMath::Pi()/6);
  double xinipos = ct.ReturnX(0, -3);
  double zinipos = ct.ReturnY(0, -3);
  double xdir = ct.ReturnX(0, 1);
  double zdir = ct.ReturnY(0, 1);
  double theta = acos(xdir/sqrt(xdir*xdir+zdir*zdir));
  double rstep, thetastep;
  if(0){
    //slow
    rstep = 1e-5;
    thetastep = 1e-7;
  }else if(0){
    //fast
    rstep = 1e-3;
    thetastep = 1e-5;
  }else{
    //normal
    rstep = 1e-4;
    thetastep = 1e-6;
  }
  double rangelimit = 1e2;
  CParticle proton = CParticleDefinition::GetParticle("proton");
  proton.k(250);

  std::vector<double> vx,vy;
  vx.push_back(xinipos);
  vy.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx, vy, theta, rstep, thetastep, rangelimit,
		    proton.m(), proton.beta(), 1);

  std::vector<double> vx2,vy2;
  vx2.push_back(xinipos);
  vy2.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx2, vy2, theta, rstep, thetastep, rangelimit,
		    2*proton.m(), proton.beta(), 1);

  std::vector<double> vx3,vy3;
  vx3.push_back(xinipos);
  vy3.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx3, vy3, theta, rstep, thetastep, rangelimit,
		    3*proton.m(), proton.beta(), 1);

  std::vector<double> vx4,vy4;
  vx4.push_back(xinipos);
  vy4.push_back(zinipos);
  CMagnet::CalTrack(map2d, vx4, vy4, theta, rstep, thetastep, rangelimit,
		    4*proton.m(), proton.beta(), 0);

  std::cout << "A/Z=1 " << vx.back() << " " << vy.back() << std::endl;
  std::cout << "A/Z=2 " << vx2.back() << " " << vy2.back() << std::endl;
  std::cout << "A/Z=3 " << vx3.back() << " " << vy3.back() << std::endl;
  std::cout << "q=0 "   << vx4.back() << " " << vy4.back() << std::endl;

  TGraph* graph = new TGraph(vx.size(), &(*vx.begin()), &(*vy.begin()));
  TGraph* graph2 = new TGraph(vx2.size(), &(*vx2.begin()), &(*vy2.begin()));
  TGraph* graph3 = new TGraph(vx3.size(), &(*vx3.begin()), &(*vy3.begin()));
  TGraph* graph4 = new TGraph(vx4.size(), &(*vx4.begin()), &(*vy4.begin()));

  fmap->Draw("colz");
  graph->SetLineColor(1);
  graph->SetLineWidth(3);
  graph->Draw("same");
  graph2->SetLineColor(28);
  graph2->SetLineWidth(3);
  graph2->Draw("same");
  graph3->SetLineColor(4);
  graph3->SetLineWidth(3);
  graph3->Draw("same");
  graph4->SetLineColor(5);
  graph4->SetLineWidth(3);
  graph4->Draw("same");

  canvas->Update();

  return 0;
}

//  std::vector<double> vxprime, vyprime;
//  std::vector<double>::iterator itx = vx.begin();
//  std::vector<double>::iterator ity = vy.begin();
//  while(itx != vx.end() || ity != vy.end()){
//    vxprime.push_back(ct.GoX(*itx, *ity));
//    vyprime.push_back(ct.GoY(*itx, *ity));
//    ++itx;++ity;
//  }
//  vx = vxprime;
//  vy = vyprime;
//  vxprime.clear();
//  vyprime.clear();
//
//  itx = vx2.begin();
//  ity = vy2.begin();
//  while(itx != vx2.end() || ity != vy2.end()){
//    vxprime.push_back(ct.GoX(*itx, *ity));
//    vyprime.push_back(ct.GoY(*itx, *ity));
//    ++itx;++ity;
//  }
//  vx2 = vxprime;
//  vy2 = vyprime;
//  vxprime.clear();
//  vyprime.clear();
//  
//  itx = vx3.begin();
//  ity = vy3.begin();
//  while(itx != vx3.end() || ity != vy3.end()){
//    vxprime.push_back(ct.GoX(*itx, *ity));
//    vyprime.push_back(ct.GoY(*itx, *ity));
//    ++itx;++ity;
//  }
//  vx3 = vxprime;
//  vy3 = vyprime;
//  vxprime.clear();
//  vyprime.clear();
//  
//  itx = vx4.begin();
//  ity = vy4.begin();
//  while(itx != vx4.end() || ity != vy4.end()){
//    vxprime.push_back(ct.GoX(*itx, *ity));
//    vyprime.push_back(ct.GoY(*itx, *ity));
//    ++itx;++ity;
//  }
//  vx4 = vxprime;
//  vy4 = vyprime;
//  vxprime.clear();
//  vyprime.clear();

