#ifndef CANALYSIS_HH
#define CANALYSIS_HH

#include "TROOT.h"

#include <vector>

class TGraph2DErrors;

typedef std::vector<double> FitPointType;

class CAnalysis
{
public:
  CAnalysis();
  ~CAnalysis();

  //least squares straight line(y=a+bx) fit
  //Statistics for nuclear and particle physicists, LOUIS LYONS
  //
  //ysigma:uncertainty for each y, output:a, b
public:
  static void LinearFit(const FitPointType& x, const FitPointType& y,
			const FitPointType& ysigma, double& a, double& b);
  //standard deviation
  static double SD(const FitPointType& x, const FitPointType& y,
		   double a, double b);
private:
  static FitPointType MultiplyVector(FitPointType x, FitPointType y);
  //p127, (5.6)
  static double SB(const FitPointType& f, const FitPointType& ysigma);
  static double SB(const FitPointType& ysigma);

  static FitPointType::size_type FitPointNumber;

  //3D fit
public:
  //3D line
  static void line(double t, double *p, double &x, double &y, double &z);
  //3次元における誤差を考慮した距離
  static double distance2_3d(double x,double y,double z, double ex, double ey, double ez, double *p);
  //  2次元(x,y)のみで距離を定義、こっちのほうがなぜか良い結果になる。
  static double distance2_2d(double x,double y,double z, double ex, double ey, double ez, double *p);
  //3次元直線との距離
  static double distance2(double x,double y,double z, double /*ex*/, double /*ey*/, double /*ez*/, double *p);
  static void SumDistance2(int &, double *, double & sum, double * par, int );
  static double line3Dfit(TGraph2DErrors* gr, double parFit[4]);
  static void DrawLine(double parFit[4], Color_t color, double t0, double dt);

  //EventLoop
  static void EventLoop();

  ClassDef(CAnalysis, 0)
};

#endif
