//
#ifndef CDataHime_h
#define CDataHime_h 1

#include <string>
#include <vector>
#include <deque>

#include "TString.h"
#include "TVector3.h"
#include "TLorentzVector.h"

#include "CData.hh"

class CDataHime : public CData
{
public:
  class OneData
  {
  public:
    OneData(std::vector<Int_t> id = std::vector<Int_t>(),
	    Double_t tl = 0., Double_t tr = 0.,
	    Double_t el = 0., Double_t er = 0., 
	    Double_t t = 0., Double_t e = 0.,
	    TVector3 x = TVector3())
      : fId(id), fTL(tl), fTR(tr), fAL(el), fAR(er), fT(t), fE(e), fX(x)
    {;}
    ~OneData(){;}

    bool operator==(const OneData& onedata)
    {
      if(this->fId == onedata.fId &&
	 this->fTL == onedata.fTL &&
	 this->fTR == onedata.fTR &&
	 this->fAL == onedata.fAL &&
	 this->fAR == onedata.fAR &&
	 this->fT == onedata.fT &&
	 this->fE == onedata.fE &&
	 this->fX == onedata.fX){
	return true;
      }else{
	return false;
      }
    }

    void Print();

    std::vector<Int_t> fId;
    Double_t fTL;
    Double_t fTR;
    Double_t fAL;
    Double_t fAR;

    Double_t fT;
    Double_t fE;
    TVector3 fX;
  };

  std::deque<CDataHime::OneData> fDequeOfOneData;
 
  TLorentzVector fX;
  TLorentzVector fP;
  TLorentzVector fX2;
  TLorentzVector fP2;

  //recoil proton
  TLorentzVector fPrp;

  void CalMomentum() throw();
  void CalMomentum2() throw();
  void ClearMomentum();

public:
  CDataHime();
  CDataHime(const char* name, const char* title = "");
  CDataHime(const TString& name, const TString& title = "");
  CDataHime(const CDataHime& himehit);
  CDataHime& operator=(const CDataHime& rhs);
  ~CDataHime();

  CDataHime::OneData* Find(Int_t idmother, Int_t id);
  void Reset();

  ClassDef(CDataHime, 1);
};

#endif


