#include <iostream>
#include <string>
#include <cstring>
#include <vector>

#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"

#include "NMadoka.hh"

TFile* NMadoka::OpenFile(std::string filename, std::string option)
{
  TFile* file = gROOT->GetFile(filename.c_str());

  if(file){
    std::cout << filename << " already exists, cd to " << filename << std::endl;
    file->cd();
  }else{
    file = TFile::Open(filename.c_str(), option.c_str());
  }

  return file;
}

TTree* NMadoka::CreateTree(std::string treename, std::string title)
{
  TTree* tree = GetTree(treename);

  if(tree){
    std::cout << treename << " already exists" << std::endl;    
  }else{
    tree = new TTree(treename.c_str(), title.c_str());
  }

  return tree;
}

TObject* NMadoka::FindObject(std::string name)
{
  TFile* file = GetFile();
  TObject* obj = 0;

  if(file){
    obj = file->FindObject(name.c_str());
  }
  
  return obj;
}

void NMadoka::Fill(std::string treename)
{
  TTree* tree = GetTree(treename);

  if(tree){
    tree->Fill();
  }else{
    std::cout << treename << " doesn't exist" << std::endl;    
  }

  return;
}

void NMadoka::Write()
{
  TFile* file = GetFile();

  if(file){
    file->Write();
  }

  return;
}

void NMadoka::CloseFile()
{
  TFile* file = GetFile();

  if(file){
    file->Close();
  }

  return;
}

TFile* NMadoka::GetFile()
{
  TFile* file = gROOT->GetFile();

  if(!file){
    std::cout << "call CreateFile first" << std::endl;
  }

  return file;
}

TTree* NMadoka::GetTree(std::string treename)
{
  TFile* file = GetFile();
  TObject* obj = 0;
  TTree* tree = 0;

  if(file){
    obj = gDirectory->FindObject(treename.c_str());
    if(obj && obj->InheritsFrom("TTree")){
      tree = (TTree*)obj;
    }
  }

  return tree;
}

