
#include "TROOT.h"
#include "TTree.h"
#include "TBranch.h"
#include "TLeaf.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TCut.h"
#include "TCutG.h"

#include "CDynamicFunction.hh"
#include "CSimpleFunction.hh"
#include "CTreeManager.hh"
#include "CBranchManager.hh"
#include "CHistManager.hh"
#include "CCutManager.hh"
#include "CCreateHist.hh"
#include "CCreateCut.hh"
#include "CFileManager.hh"
#include "CCanvasManager.hh"
#include "CMyCalculator.hh"
#include "NConverter.hh"

#include "CInitializer.hh"
#include "CGetter.hh"

#include "Alias.hh"

CHistGetter gHist;
CCutGetter gCut;
CCutGGetter gCutg;
CCanvasGetter gCanvas;

namespace NAlias
{
  CInitializer* initializer = new CInitializer;
}

namespace NAlias
{
  CDynamicFunction* dynamicfunction;
  CSimpleFunction* simplefunction;
  CCalculator* calculator;
  CFileManager* filemanager;
  CTreeManager* treemanager;
  CBranchManager* branchmanager;
  CHistManager* histmanager;
  CCutManager* cutmanager;
  CCreateHist* createhist;
  CCreateCut* createcut;
  CCanvasManager* canvasmanager;
}

//alias of alias
void fetch(std::string filename, int offset)
{
  readhist(filename, offset);
  hp();
  ls();
}
void hstore(std::string filename)
{
  writehist(filename);
}
void zone(Int_t nx, Int_t ny, Float_t xmargin, Float_t ymargin, Int_t color)
{
  divide(nx, ny, xmargin, ymargin, color);
}
void ls(){histls();}
void ht(std::string name)
{
  ncd();
  histht(name);
}
void ht(int i)
{
  ncd();
  histht(i);
}
void ht(int id_begin, int id_end)
{
  histht(id_begin, id_end);
}
void hp()
{
  ncd();
  histhp();
}
void hb()
{
  ncd();
  histhb();
}
void hn()
{
  ncd();  
  histhn();
}

//simple function
void clear()
{
  NAlias::cutmanager->Clear();
  NAlias::histmanager->Clear();  
  NAlias::treemanager->Clear();
  NAlias::branchmanager->Clear();
  NAlias::canvasmanager->Clear();
  NAlias::filemanager->Clear();
  NAlias::initializer->Clear();
}

//CDynamicFunction
void xval(){NAlias::dynamicfunction->XValue();}
void yval(){NAlias::dynamicfunction->YValue();}
void xyval(){NAlias::dynamicfunction->XYValue();}
//void circle(){NAlias::dynamicfunction->Circle();}
void zoom(){NAlias::dynamicfunction->WheelZoom();}

//CSimpleFunction
TFitResultPtr fit(const char* formula, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax){return NAlias::simplefunction->Fit(formula, option, goption, xmin, xmax);}
TFitResultPtr fitp(int n, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax){return NAlias::simplefunction->FitPolynomial(n, option, goption, xmin, xmax);}
TFitResultPtr fite(Option_t* option, Option_t* goption, Double_t xmin,  Double_t xmax){return NAlias::simplefunction->FitExponential(option, goption, xmin, xmax);}
TFitResultPtr fitg(Option_t* option, Option_t* goption, Double_t xmin,  Double_t xmax){return NAlias::simplefunction->FitGaussian(option, goption, xmin, xmax);}
TFitResultPtr fitl(Option_t* option, Option_t* goption, Double_t xmin,  Double_t xmax){return NAlias::simplefunction->FitLandau(option, goption, xmin, xmax);}

TProfile* pfx(Int_t firstybin, Int_t lastybin, Option_t* option){return NAlias::simplefunction->ProfileX(firstybin, lastybin, option);}
TProfile* pfy(Int_t firstxbin, Int_t lastxbin, Option_t* option){return NAlias::simplefunction->ProfileY(firstxbin, lastxbin, option);}
TH1D* prx(Int_t firstybin, Int_t lastybin, Option_t* option){return NAlias::simplefunction->ProjectionX(firstybin, lastybin, option);}
TH1D* pry(Int_t firstxbin, Int_t lastxbin, Option_t* option){return NAlias::simplefunction->ProjectionY(firstxbin, lastxbin, option);}
TH1D* bnx(double firstx, double lastx, Option_t* option){return NAlias::simplefunction->BanX(firstx, lastx, option);}
TH1D* bny(double firsty, double lasty, Option_t* option){return NAlias::simplefunction->BanY(firsty, lasty, option);}

//ICalculator
void calculate(std::string treename, std::string treetitle){NAlias::calculator->Calculate(treename, treetitle);}

//CCanvasManager
void setcanvas(TCanvas* c){NAlias::canvasmanager->SetCanvas(c);}
TCanvas* getcanvas(){return NAlias::canvasmanager->GetCanvas();}
void lgx(){NAlias::canvasmanager->SetLogx(1);}
void lgy(){NAlias::canvasmanager->SetLogy(1);}
void lgz(){NAlias::canvasmanager->SetLogz(1);}
void lnx(){NAlias::canvasmanager->SetLogx(0);}
void lny(){NAlias::canvasmanager->SetLogy(0);}
void lnz(){NAlias::canvasmanager->SetLogz(0);}
void divide(Int_t nx, Int_t ny, Float_t xmargin, Float_t ymargin, Int_t color)
{NAlias::canvasmanager->Divide(nx, ny, xmargin, ymargin, color);}
void cd(Int_t subpadnumber){NAlias::canvasmanager->cd(subpadnumber);}
void ncd(){NAlias::canvasmanager->NextSubpad();}
void size(UInt_t ww, UInt_t wh){NAlias::canvasmanager->Resize(ww, wh);}
void size(){NAlias::canvasmanager->Resize();}
void print(std::string filename, Option_t* option){NAlias::canvasmanager->SaveAs(filename, option);}

//CFileManager
void readtree(std::string filename, std::string treename){NAlias::filemanager->ReadTree(filename, treename);}
void writetree(std::string filename, std::string option){NAlias::filemanager->WriteTree(filename, option);}
void readhist(std::string filename, int offset){NAlias::filemanager->ReadHist(filename, offset);}
void writehist(std::string filename, std::string option){NAlias::filemanager->WriteHist(filename, option);}
void txttotree(std::string filename, std::string branchname, std::string treename, std::string treetitle){NAlias::filemanager->TxtToTree(filename, branchname, treename, treetitle);}
void vectortotree(std::vector<std::vector<int> >& eventbuffer, std::string branchname, std::string treename, std::string treetitle){NAlias::filemanager->VectorToTree(eventbuffer, branchname, treename, treetitle);}
TFile* getfile(std::string filename){return NAlias::filemanager->GetFile(filename);}
TFile* getfile(int id){return NAlias::filemanager->GetFile(id);}
TFile* getfile(){return NAlias::filemanager->GetFile();}
void filels(){NAlias::filemanager->PrintIdNameTitle();}

//CTreeManager
TTree* gettree(std::string name){return NAlias::treemanager->GetTree(name);}
TTree* gettree(int id){return NAlias::treemanager->GetTree(id);}
TTree* gettree(){return NAlias::treemanager->GetTree();}
void treels(){NAlias::treemanager->PrintIdNameTitle();}
void befriends(){NAlias::treemanager->BeFriends();}
void treetobranch(){NAlias::treemanager->TreeToBranch();}

//CBranchManager
TBranch* getbranch(std::string name){return NAlias::branchmanager->GetBranch(name);}
TBranch* getbranch(int id){return NAlias::branchmanager->GetBranch(id);}
TBranch* getbranch(){return NAlias::branchmanager->GetBranch();}
TLeaf* getleaf(std::string name){return NAlias::branchmanager->GetLeaf(name);}
TLeaf* getleaf(int id){return NAlias::branchmanager->GetLeaf(id);}
TLeaf* getleaf(){return NAlias::branchmanager->GetLeaf();}
void pushbranch(std::string name, std::string title){NAlias::branchmanager->IArrayManager::Push(name, title);}
void branchls(){NAlias::branchmanager->PrintIdNameTitle();}
void branchht(std::string name, const TCut* selection){NAlias::branchmanager->Draw(name, selection);}
void branchht(int idx, int idy, int idz, const TCut* selection){NAlias::branchmanager->Draw(idx, idy, idz, selection);}
void branchht(int idx, int idy, const TCut* selection){NAlias::branchmanager->Draw(idx, idy, selection);}
void branchht(int idx, const TCut* selection){NAlias::branchmanager->Draw(idx, selection);}
void branchhp(){NAlias::branchmanager->DrawCurrent();}
void branchhb(){NAlias::branchmanager->DrawPrevious();}
void branchhn(){NAlias::branchmanager->DrawNext();}

//CHistManager
void clearhist(){NAlias::histmanager->Clear();}
void erasehist(int id){NAlias::histmanager->Erase(id);}
void erasehist(){NAlias::histmanager->Erase();}
void erasehist(int ids, int idf){NAlias::histmanager->Erase(ids, idf);}
TH1* gethist(std::string name){return NAlias::histmanager->GetHist(name);}
TH1* gethist(int id){return NAlias::histmanager->GetHist(id);}
TH1* gethist(){return NAlias::histmanager->GetHist();}
void inserthist(TFile* f, int offset){NAlias::histmanager->Insert(f, offset);}
void histht(std::string name){NAlias::histmanager->Draw(name);}
void histht(int i){NAlias::histmanager->Draw(i);}
void histht(int id_begin, int id_end){NAlias::histmanager->DrawSequential(id_begin, id_end);}
void histhp(){NAlias::histmanager->DrawCurrent();}
void histhb(){NAlias::histmanager->DrawPrevious();}
void histhn(){NAlias::histmanager->DrawNext();}
void histls(){NAlias::histmanager->PrintIdNameTitle();}
void setdrawoption(std::string foption){NAlias::histmanager->SetDrawOption(foption);}
void setdrawoption(){NAlias::histmanager->SetDrawOption();}
void applyoption(std::string name){NAlias::histmanager->ApplyOption(name);}
void applyoption(int id){NAlias::histmanager->ApplyOption(id);}
void applyoption(){NAlias::histmanager->ApplyOption();}
void applyoptionall(){NAlias::histmanager->ApplyOptionAll();}
void setfillcolor(Color_t fcolor){NAlias::histmanager->SetFillColor(fcolor);}
void setfillcolor(){NAlias::histmanager->SetFillColor();}

//CCutManager
TCut* getcut(std::string name){return NAlias::cutmanager->GetCut(name);}
TCut* getcut(int id){return NAlias::cutmanager->GetCut(id);}
TCut* getcut(){return NAlias::cutmanager->GetCut();}
TCutG* getcutg(std::string name){return NAlias::cutmanager->GetCutG(name);}
TCutG* getcutg(int id){return NAlias::cutmanager->GetCutG(id);}
TCutG* getcutg(){return NAlias::cutmanager->GetCutG();}
void cutls(){NAlias::cutmanager->PrintIdNameTitle();}

//CCreateHist
void createhist(int idx,
		Int_t nbinsx,
		Double_t xlow,
		Double_t xup,
		int idy,
		Int_t nbinsy,
		Double_t ylow,
		Double_t yup,
		int idz,
		Int_t nbinsz,
		Double_t zlow,
		Double_t zup,
		std::string histname,
		const TCut* selection)
{
  NAlias::createhist->CreateHistFromBranch(idx, nbinsx, xlow, xup,
					   idy, nbinsy, ylow, yup,
					   idz, nbinsz, zlow, zup, histname, selection);
}
void createhist(int idx,
		Int_t nbinsx,
		Double_t xlow,
		Double_t xup,
		int idy,
		Int_t nbinsy,
		Double_t ylow,
		Double_t yup,
		std::string histname,
		const TCut* selection)
{
  NAlias::createhist->CreateHistFromBranch(idx, nbinsx, xlow, xup,
					   idy, nbinsy, ylow, yup, histname, selection);
}
void createhist(int idx,
		Int_t nbinsx,
		Double_t xlow,
		Double_t xup,
		std::string histname,
		const TCut* selection)
{
  NAlias::createhist->CreateHistFromBranch(idx, nbinsx, xlow, xup, histname, selection);
}
void createhist(std::string namex,
		std::string namey,
		std::string namez,
		std::string histname,
		const TCut* selection)
{
  NAlias::createhist->CreateHistFromBranch(namex, namey, namez, histname, selection);
}
void createhist(int idx,
		int idy,
		int idz,
		std::string histname,
		const TCut* selection)
{
  NAlias::createhist->CreateHistFromBranch(idx, idy, idz, histname, selection);
}
void createhistall(){NAlias::createhist->CreateHistFromBranchAll();}

//CCreateCut
void createcut(std::string name, std::string title){NAlias::createcut->CreateCut(name, title);}
void createcut(std::string name, TCut* cut){NAlias::createcut->CreateCut(name, cut);}
void createcutg(std::string name, std::string title){NAlias::createcut->CreateCutG(name, title);}
void circlecut(std::string name, std::string histname){NAlias::createcut->CircleCut(name, histname);}

//Converter
void convertrdftotxt(std::string InputFileName, std::string OutputFileName)
{NConverter::ConvertRdfToTxt(InputFileName, OutputFileName);}
void convertrdftovector(std::string InputFileName, std::vector<std::vector<int> >& eventbuffer)
{NConverter::ConvertRdfToVector(InputFileName, eventbuffer);}
