#ifndef IARRAYMANAGER_HH
#define IARRAYMANAGER_HH

#include "TFile.h"
#include "TList.h"

#include <sstream>
#include <string>

#include "NGlobal.hh"

#include "CdequeIdNameTitle.hh"

class CCanvasManager;
class CCreateHist;
class CCreateCut;

namespace NAlias
{
  extern CCanvasManager* canvasmanager;
}

class IArrayManager
{
public:
  virtual void Draw(int) = 0;
private:
  virtual std::string FunctionName(std::string functionname) const = 0;

public:
  IArrayManager() : idnametitle(), current(idnametitle.end()), nextid(0), defword("-->"), listname("idnametitle"){
    fList = new TList;
}
  virtual ~IArrayManager(){;}

  virtual void Insert(std::string name, std::string title, int order, int id = -1);
  virtual void InsertAfterCurrent(std::string name, std::string title);
  virtual void Push(std::string name, std::string title);
  virtual CdequeIdNameTitle::iterator Erase(int id);
  virtual CdequeIdNameTitle::iterator Erase(std::string name);
  virtual CdequeIdNameTitle::iterator Erase();
  virtual CdequeIdNameTitle::iterator Erase(int id_begin, int id_end);
  virtual CdequeIdNameTitle::iterator GetCurrent();
  virtual CdequeIdNameTitle::iterator SetCurrent(CdequeIdNameTitle::iterator it);
  virtual void PushToList(TObject* obj){fList->Add(obj);}
  virtual TList* GetList(){return fList;};  
  virtual int GetId(std::string name);
  virtual int GetId();
  virtual std::string GetName(std::string name);
  virtual std::string GetName(int id);
  virtual std::string GetName();
  virtual std::string GetTitle(std::string name);
  virtual std::string GetTitle(int id);
  virtual std::string GetTitle();
  virtual void DrawSequential(int id_begin, int id_end);
  virtual void DrawCurrent();
  virtual void DrawPrevious();
  virtual void DrawNext();
  virtual void Clear();

  virtual void PrintIdName();
  virtual void PrintIdNameTitle();

  CdequeIdNameTitle::iterator FindById(int id);
  CdequeIdNameTitle::iterator FindByName(std::string name);
  CdequeIdNameTitle::iterator FindByTitle(std::string title);

  bool ExistId(int id);
  bool ExistName(std::string name);
  bool ExistTitle(std::string title);

  int GetNextId() const;
  int SetNextId(int id);
  void PushNextId();
  void ResetNextId();

  CdequeIdNameTitle::size_type Size();
  CdequeIdNameTitle::iterator Begin();
  CdequeIdNameTitle::iterator End();
  CdequeIdNameTitle::reverse_iterator RBegin();
  CdequeIdNameTitle::reverse_iterator REnd();

  bool Check(EMessageLevel level = kMID) const;
  bool Check(int id, EMessageLevel level = kMID) const;
  bool Check(std::string str, EMessageLevel level = kMID) const;
  bool Check(CdequeIdNameTitle::iterator it, EMessageLevel level = kMID) const;
  template <class T>
  bool Check(T* ptr) const;

protected:
  CdequeIdNameTitle idnametitle;
  CdequeIdNameTitle::iterator current;
  TList* fList;

  int nextid;
  virtual std::string Word(int wordflag = 0);
  std::string defword;
  std::string listname;
};

template <class T>
inline bool IArrayManager::Check(T* ptr) const
{
  if(!ptr){
    std::ostringstream os;
    os << FunctionName("Check") << "not valid pointer";
    NGlobal::Message(os.str(), kMID);

    return false;
  }else{
    return true;
  }
}

template <>
inline bool IArrayManager::Check<TFile>(TFile* file) const
{
  if(!file || file->IsZombie()) {
    std::ostringstream os;
    os << FunctionName("Check") << "error opening file";
    NGlobal::Message(os.str(), kMID);

    return false;
  }else{
    return true;
  }
}


#endif
