#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TChain.h"
#include "TBranch.h"

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>

#include "CInitializer.hh"
#include "CBranchManager.hh"
#include "CHistManager.hh"
#include "NGlobal.hh"

#include "CFileManager.hh"

void CFileManager::ReadTree(std::string filename, std::string treename, std::string chainname)
{
  if(treename.empty()){
    std::cout << "no treename is not yet, input treename like CFileManager::ReadTree(\"filename\", \"T\")" << std::endl;;
    return;
  }

  TFile* file = new TFile(filename.c_str());
  if(!Check(file)) return;
  //  NAlias::initializer->cd();

  TKey* key = file->FindKey(treename.c_str());
  if(!key){
    delete file;

    std::ostringstream os;
    os << FunctionName("ReadTree") << "cannot find " << treename << " in " << filename << '\n';
    NGlobal::Message(os.str(), kMID);

    return;
  }

  PushToList(file);
  Push(filename, "read:tree");

  if(chainname.empty()) chainname = treename;
  TChain* chain;
  TObject* obj = gROOT->FindObjectAny(chainname.c_str());
  if(obj && obj->InheritsFrom("TChain")){
    chain = (TChain*)obj;
    chain->AddFile(filename.c_str(), 0, treename.c_str());

    std::ostringstream os;
    os << FunctionName("ReadTree") << chain->GetNtrees() << " trees are loaded sequentially";
    NGlobal::Message(os.str(), kMID);
  }else{
    chain = new TChain(chainname.c_str());
    PushToList(chain);
    //  chain->Add(filename.c_str(), (Long64_t)TChain::kBigNumber, treename.c_str());
    chain->AddFile(filename.c_str(), 0, treename.c_str());
    NAlias::treemanager->Push(treename, filename);

    std::ostringstream os;
    os << FunctionName("ReadTree") << " new chain is created, name:" << treename;
    NGlobal::Message(os.str(), kMID);
  }    

  std::ostringstream os;
  os << FunctionName("ReadTree") << "successfully read in " << filename;
  NGlobal::Message(os.str(), kMID);
}

void CFileManager::WriteTree(std::string filename, std::string option)
{
  TFile* file = new TFile(filename.c_str(), option.c_str());
  if(!Check(file)){
    delete file;
    return;
  }
  PushToList(file);
  Push(filename, "write:tree");  

  NAlias::treemanager->Write();
  //  NAlias::initializer->cd();

  std::ostringstream os;
  os << FunctionName("WriteTree") << "write tree to " << filename;
  NGlobal::Message(os.str(), kMID);
}

void CFileManager::ReadHist(std::string filename, int offset)
{
  TFile* file = new TFile(filename.c_str());
  if(!Check(file)){
    delete file;
    return;
  }
  //  NAlias::initializer->cd();
  PushToList(file);
  Push(file->GetName(), "read:hist");

  NAlias::histmanager->Insert(file, offset);

  std::ostringstream os;
  os << FunctionName("ReadTree") << "successfully read in " << filename;
  NGlobal::Message(os.str(), kMID);
}

void CFileManager::WriteHist(std::string filename, std::string option)
{
  TFile* file = new TFile(filename.c_str(), option.c_str());
  if(!Check(file)){
    delete file;
    return;
  }
  PushToList(file);
  Push(filename, "write:hist");  
  
  NAlias::histmanager->Write();
  //  NAlias::initializer->cd();

  std::ostringstream os;
  os << FunctionName("WriteHist") << "write hist to " << filename;
  NGlobal::Message(os.str(), kMID);
}

void CFileManager::TxtToTree(std::string filename, std::string branchname, std::string treename, std::string treetitle)
{
  std::ifstream ifs(filename.c_str());
  if(ifs.bad() || ifs.fail()){
    std::cout << "file open error" << std::endl;
    return;
  }
  Push(filename, "read:dat");  

  TObject* obj = gROOT->FindObjectAny(treename.c_str());
  TTree* tree;
  if(obj && obj->InheritsFrom("TTree")){
    NAlias::treemanager->Erase(treename);
    NAlias::branchmanager->Clear();
    tree = (TTree*)obj;
  }else{
    tree = new TTree(treename.c_str(), treetitle.c_str());
  }

  double leaf;
  std::string leaflist = branchname + "/D";
  TBranch* branch = tree->Branch(branchname.c_str(), &leaf, leaflist.c_str());

  double xbuf;
  ifs >> xbuf;
  while(!ifs.eof()){
    leaf = xbuf;
    //    tree->Fill();
    branch->Fill();

    ifs >> xbuf;
  }
  tree->SetEntries(branch->GetEntries());
  tree->Print();
  NAlias::treemanager->Push(tree->GetName(), "onmemory");
}

void CFileManager::VectorToTree(std::vector<std::vector<int> >&  eventbuffer, std::string branchname, std::string treename, std::string treetitle)
{
  if(eventbuffer.empty()){
    std::cout << "vector is empty" << std::endl;
    return;
  }

  TObject* obj = gROOT->FindObjectAny(treename.c_str());
  TTree* tree;
  if(obj && obj->InheritsFrom("TTree")){
    NAlias::treemanager->Erase(treename);
    NAlias::branchmanager->Clear();
    tree = (TTree*)obj;
  }else{
    tree = new TTree(treename.c_str(), treetitle.c_str());
  }

  std::vector<int*> leaf_v;
  std::vector<TBranch*> branch_v;
  for(std::vector<std::vector<int> >::size_type i = 0; i < eventbuffer[0].size(); ++i){
    leaf_v.push_back(new int);

    std::ostringstream oss;
    oss << branchname << i;
    std::string branchname_i = oss.str();
    oss << "/I";
    branch_v.push_back(tree->Branch(branchname_i.c_str(), leaf_v[i], oss.str().c_str()));
  }

  for(std::vector<std::vector<int> >::size_type j = 0; j < eventbuffer.size(); ++j){
    for(std::vector<std::vector<int> >::size_type i = 0; i < eventbuffer[0].size(); ++i){
      *leaf_v[i] = eventbuffer[j][i];
      branch_v[i]->Fill();
    }
  }

  tree->SetEntries(branch_v[0]->GetEntries());
  tree->Print();
  NAlias::treemanager->Push(tree->GetName(), "onmemory");
}

TFile* CFileManager::GetFile(std::string filename)
{
  TFile* file = gROOT->GetFile(filename.c_str());
  if(!Check(file)) return 0;

  return file;
}

TFile* CFileManager::GetFile(int id)
{
  std::string name = GetName(id);
  if(!Check(name)) return 0;

  TFile* file = GetFile(name);
  if(!Check(file)) return 0;

  return file;
}

TFile* CFileManager::GetFile()
{
  std::string name = GetName();  
  if(!Check(name)) return 0;

  TFile* file = GetFile(name);
  if(!Check(file)) return 0;

  return file;
}

void CFileManager::Draw(int)
{
  /* nothing */
}

std::string CFileManager::FunctionName(std::string functionname) const
{
  return "[CFileManager::" + functionname + "]$ ";
}
