
#include <iostream>
#include <deque>
#include <fstream>
#include <sstream>
#include <cmath>

#include "TF1.h"

#include "CMapFunction1D.hh"
#include "FFunction.hh"

ClassImp(CMapFunction1D)

CMapFunction1D::CMapFunction1D() : CMapFunction()
{;}

CMapFunction1D::~CMapFunction1D()
{;}

int CMapFunction1D::SetFile(std::string filename)
{
  return SetFile(filename,0,1);
}

int CMapFunction1D::SetFile(std::string filename,
			    double offsetx, double intervalx)
{
  std::ifstream ifs(filename.c_str());
  if(!ifs.is_open()){
        std::cout << "cannot open file" << std::endl;
	return 0;
  }

  m_offsetx = offsetx;
  m_intervalx = intervalx;

  while(1){
    std::string str;
    std::getline(ifs, str);
    if(ifs.eof()) break;
    if(str.find("#") != std::string::npos) continue;
    std::istringstream iss(str);

    double xtemp;
    while(iss >> xtemp){
      m_x.push_back(xtemp);
    }
  }

  m_xmin = m_offsetx;
  m_xmax = m_offsetx+(m_x.size()-1)*m_intervalx;

  fMapFunction = new TF1("mapfunction1d", this,
			 static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			 (&CMapFunction1D::MapFunction),
			 m_xmin, m_xmax, 0, "CMapFunction1D", "MapFunction");

  return 1;
}

CMapFunction::type_map CMapFunction1D::MapFunction(Double_t x)
{
  if(x < m_xmin || x >= m_xmax) return 0;

  double xx = (x-m_offsetx)/m_intervalx;

  int floor_xx = xx;
  int ceil_xx = floor_xx+1;
  
  double sub_xx = xx-floor_xx;
  double rsub_xx = 1-sub_xx;

  type_map val = (MapFunction(ceil_xx)*sub_xx + MapFunction(floor_xx)*rsub_xx);

  return val;
}

//direct access
CMapFunction::type_map CMapFunction1D::MapFunction(int i)
{
  return m_x[i];
}

Double_t CMapFunction1D::MapFunction(Double_t* x, Double_t* /*par*/)
{
  return MapFunction(x[0]);
}

void CMapFunction1D::Print()
{
  std::copy(m_x.begin(), m_x.end(), std::ostream_iterator<double>(std::cout, "\n"));
}

Double_t CMapFunction1D::VRMF1D(Double_t xx, Double_t k, Double_t a, Double_t r0,
			      Double_t Eb, Double_t ma, Double_t mb)
{
  return k*VirtualResonance(xx, a, r0, Eb, ma, mb)*MapFunction(xx);
}

Double_t CMapFunction1D::VRMF1D(Double_t* x, Double_t* par)
{
  Double_t xx = x[0];
  Double_t k = par[0];
  Double_t a = par[1];
  Double_t r0 = par[2];
  Double_t Eb = par[3];
  Double_t ma = par[4];
  Double_t mb = par[5];

  return VRMF1D(xx, k, a, r0, Eb, ma, mb);
}

Double_t CMapFunction1D::BWMF1D(Double_t xx, Double_t k,
				Double_t er, Double_t gamma)
{
  return k*BrietWigner(xx, er, gamma)*MapFunction(xx);
}

Double_t CMapFunction1D::BWMF1D(Double_t* x, Double_t* par)
{
  Double_t xx = x[0];
  Double_t k = par[0];
  Double_t er = par[1];
  Double_t gamma = par[2];

  return BWMF1D(xx, k, er, gamma);
}

Double_t CMapFunction1D::BW2MF1D(Double_t xx, Double_t k,
				 Double_t k1, Double_t er1, Double_t gamma1,
				 Double_t er2, Double_t gamma2)
{
  return k*(k1*BrietWigner(xx, er1, gamma1)+BrietWigner(xx, er2, gamma2))
    *MapFunction(xx);
}

Double_t CMapFunction1D::BW2MF1D(Double_t* x, Double_t* par)
{
  Double_t xx = x[0];
  Double_t k = par[0];
  Double_t k1 = par[1];
  Double_t er1 = par[2];
  Double_t gamma1 = par[3];
  Double_t er2 = par[4];
  Double_t gamma2 = par[5];

  return BW2MF1D(xx, k, k1, er1, gamma1, er2, gamma2);
}

Double_t CMapFunction1D::VRBW2MF1D(Double_t xx, Double_t k,
				   Double_t a, Double_t r0, Double_t Eb, Double_t ma, Double_t mb,
				   Double_t k1, Double_t er1, Double_t gamma1,
				   Double_t k2, Double_t er2, Double_t gamma2)
{
  return k*(VirtualResonance(xx, a, r0, Eb, ma, mb)
	    +k1*BrietWigner(xx, er1, gamma1)
	    +k2*BrietWigner(xx, er2, gamma2))
    *MapFunction(xx);
}

Double_t CMapFunction1D::VRBW2MF1D(Double_t* x, Double_t* par)
{
  Double_t xx = x[0];
  Double_t k = par[0];

  Double_t a = par[1];
  Double_t r0 = par[2];
  Double_t Eb = par[3];
  Double_t ma = par[4];
  Double_t mb = par[5];

  Double_t k1 = par[6];
  Double_t er1 = par[7];
  Double_t gamma1 = par[8];

  Double_t k2 = par[9];
  Double_t er2 = par[10];
  Double_t gamma2 = par[11];

  return VRBW2MF1D(xx, k, a, r0, Eb, ma, mb, k1, er1, gamma1, k2, er2, gamma2);

}
