
#include <iostream>
#include <iomanip>
#include <iterator>
#include <fstream>
#include <sstream>
#include <deque>
#include <cmath>

#include <cstdlib>
#include <cmath>

#include "TF1.h"
#include "TF2.h"

#include "CMapFunction2D.hh"
#include "FFunction.hh"

ClassImp(CMapFunction2D)

CMapFunction2D::CMapFunction2D() : CMapFunction()
{;}

CMapFunction2D::~CMapFunction2D()
{;}

int CMapFunction2D::SetFile(std::string filename)
{
  return SetFile(filename,0,1,0,1);
}

int CMapFunction2D::SetFile(std::string filename,
			    double offsetx, double intervalx,
			    double offsety, double intervaly)
{
  std::ifstream ifs(filename.c_str());
  if(!ifs.is_open()){
        std::cout << "cannot open file" << std::endl;
	return 0;
  }

  type_map2d xy;
  while(1){
    std::string str;
    std::getline(ifs, str);
    if(ifs.eof()) break;
    if(str.find("#") != std::string::npos) continue;
    std::istringstream iss(str);

    type_map1d x;
    double xtemp;
    while(iss >> xtemp){
      x.push_back(xtemp);
    }
    xy.push_back(x);
  }

  return SetFile(xy, offsetx, intervalx, offsety, intervaly);
}

int CMapFunction2D::SetFile(CMapFunction::type_map2d xy,
			    double offsetx, double intervalx,
			    double offsety, double intervaly)
{
  m_xy = xy;
  m_offsetx = offsetx;
  m_offsety = offsety;
  m_intervalx = intervalx;
  m_intervaly = intervaly;

  m_xmin = m_offsetx;
  m_xmax = m_offsetx+((m_xy.begin())->size()-1)*m_intervalx;
  m_ymin = m_offsety;
  m_ymax = m_offsety+(m_xy.size()-1)*m_intervaly;

  fMapFunction = new TF2("mapfunction2d", this,
			 static_cast<Double_t (CMapFunction2D::*)(Double_t*, Double_t*)>
			 (&CMapFunction2D::MapFunction),
			 m_xmin, m_xmax, m_ymin, m_ymax, 0, "CMapFunction2D", "MapFunction");

  return 1;
}

Double_t CMapFunction2D::MapFunction(Double_t* x, Double_t* /*par*/)
{
  Double_t xx = x[0];
  Double_t yy = x[1];
  
  return MapFunction(xx, yy);
}

//acccess by value
CMapFunction::type_map CMapFunction2D::MapFunction(Double_t x, Double_t y)
{
  if(x < m_xmin || x >= m_xmax || y < m_ymin || y >= m_ymax) return 0;

  double xx = (x-m_offsetx)/m_intervalx;
  double yy = (y-m_offsety)/m_intervaly;

  int floor_xx = xx;
  int floor_yy = yy;
  int ceil_xx = floor_xx+1;
  int ceil_yy = floor_yy+1;
  
  double sub_xx = xx-floor_xx;
  double sub_yy = yy-floor_yy;
  double rsub_xx = 1-sub_xx;
  double rsub_yy = 1-sub_yy;

  type_map val = (MapFunction(ceil_xx, ceil_yy)*sub_xx*sub_yy
		  + MapFunction(ceil_xx, floor_yy)*sub_xx*rsub_yy
		  + MapFunction(floor_xx, ceil_yy)*rsub_xx*sub_yy
		  + MapFunction(floor_xx, floor_yy)*rsub_xx*rsub_yy);

  return val;
}

//direct access
CMapFunction::type_map CMapFunction2D::MapFunction(int i, int j)
{
  return (m_xy[j])[i];
}

void CMapFunction2D::Print()
{
  type_map2d::const_iterator it2 = m_xy.begin();
  while(it2 != m_xy.end()){
    type_map1d::const_iterator it = it2->begin();
    while(it != it2->end()){
      std::cout << std::setw(4) << *it << " ";
      ++it;
    }
    std::cout << std::endl;
    ++it2;
  }

}
