#ifndef CPARTICLE_HH
#define CPARTICLE_HH

#include <iostream>
#include <string>

#include "TROOT.h"
#include "TLorentzVector.h"

class CParticle
{
public:
  CParticle(double mass, std::string name = "particle");
  ~CParticle();

public:
  //set function
  void p(double p0, double p1, double p2);
  void pabs(double momentum);
  void e(double energy);
  void k(double kineticenergy);
  void thetaphi(double ltheta, double lphi);
  void thetaphibar(double ltheta, double lphi);
  void beta(double b0, double b1, double b2);
  void betaabs(double beta);
  void c(double lightvelocity);
  //get funciton
  double p(int i) const;
  double p() const;
  double p2() const;
  double m() const;
  double m2() const;
  double e() const;
  double e2() const;
  double k() const;
  double k2() const;
  double theta() const;
  double phi() const;
  double thetabar() const;
  double phibar() const;
  double beta(int i) const;
  double beta() const;
  double gamma(int i) const;
  double gamma() const;
  double v(int i) const;
  double v() const;
  double c() const;
  double c2() const;

public:
  void SetMomentum(double px, double py, double pz);
  void SetMomentum(double momentum);  

  void SetEnergy(double energy);
  void SetKineticEnergy(double kineticenergy);  
  void SetThetaPhi(double ltheta, double lphi);
  void SetThetaPhiBar(double ltheta, double lphi);
  void SetBeta(double bx, double by, double bz);  
  void SetBeta(double beta);  
  void SetC(double lightvelocity);

  double GetMass() const;
  double GetMomentum() const;
  double GetMomentum(int i) const;
  double GetEnergy() const;
  double GetKineticEnergy() const;
  double GetTheta() const;
  double GetPhi() const;
  double GetThetaBar() const;
  double GetPhiBar() const;
  double GetBeta() const;
  double GetBeta(int i) const;
  double GetGamma() const;
  double GetGamma(int i) const;
  double GetVelocity() const;
  double GetVelocity(int i) const;

  double GetC() const;

  std::string GetName() const;

  void Print() const;

private:
  double ThetaBar(double ltheta) const;
  double PhiBar(double lphi) const;

private:
  TLorentzVector fP;
  std::string fName;

  double fC;

  ClassDef(CParticle, 0)
};

#endif
